/* file: "gambit.h" */

/*
 * Copyright (C) 1994-1996 by Marc Feeley, All Rights Reserved.
 */

#ifndef ___GAMBIT_H
#define ___GAMBIT_H

#ifndef ___VERSION
@error "___VERSION has not been set by includer"
#endif

#if ___VERSION != 15
@error "Incompatible version of gambit.h include file"
#endif

/*---------------------------------------------------------------------------*/

/*
 * DETERMINE PROGRAMMING ENVIRONMENT'S CHARACTERISTICS
 *
 * The following symbols need to be defined to reflect the
 * programming environment's characteristics:
 *
 *  define one of the following symbols depending on the processor type
 *    ___CPU_alpha  DEC Alpha AXP processor
 *    ___CPU_hppa   HP PA-RISC processor
 *    ___CPU_m68k   Motorola 68k family processor
 *    ___CPU_mips   MIPS Rx000 family processor
 *    ___CPU_ppc    PowerPC processor
 *    ___CPU_rs6k   IBM RS6000 processor
 *    ___CPU_sparc  SUN SPARC processor
 *    ___CPU_x86    Intel x86 family processor
 *
 *  define ___CPU_BIGEND if integers are stored in "big-endian" format
 *  define ___CPU_LITEND if integers are stored in "little-endian" format
 *
 *  define ___FORCE_32 to force the use of 32 bit words
 *
 *  define ___o as the extension for object files (e.g. "o", "obj")
 *  define ___CC_O as the command that compiles a C file to an object file
 *  define ___CC_O_SH like ___CC_O except that object file is for a shared lib
 *  define ___CLIBS as the libraries to use (e.g. "-lm", "-ldl")
 *  define ___CC_PROG1, ___CC_PROG2, and ___CC_PROG3 as the command that
 *            invokes the C compiler for a dynamic Scheme compilation
 *  define ___LD_PROG1, ___LD_PROG2, and ___LD_PROG3 as the command that
 *            invokes the linker for a dynamic Scheme compilation
 *  define ___NO_DYNAMIC_IMPORTS if the dynamic loader can not load
 *            files that import procedures defined in other files
 *  define ___SETDLPATH as the variable definition that sets up the path
 *            to search for shared libraries in "../lib"
 */

#ifdef sparc
#define ___CPU_sparc
#define ___CPU_BIGEND
#ifdef __SVR4
#define ___o "o"
#ifdef __GNUC__
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "gcc -fpic -O1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB_SH) -o"
#define ___LINK_SH_INSTALL "gcc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "gcc -shared -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "gcc -fPIC -O1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#else
#define ___CC_O "cc -xO1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "cc -KPIC -xO1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "cc \$(OBJECTS) ../lib/\$(GAMBCLIB_SH) -o"
#define ___LINK_SH_INSTALL "cc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "cc -G -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "cc -Kpic -xO1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#endif
#define ___CLIBS "-lm -ldl"
#define ___GAMBCLIB "libgambc.a"
#define ___GAMBCLIB_SH "libgambc.so.1.1"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "/usr/ccs/bin/ld -G -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#else
#define ___o "o"
#ifdef __GNUC__
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___CC_PROG1 "gcc -fPIC -O1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#else
#define ___CC_O "cc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___CC_PROG1 "cc -pic -O1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#endif
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "echo"
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "/usr/bin/ld -shared -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#endif
#endif

#ifdef sgi
#define ___CPU_mips
#define ___CPU_BIGEND
#define ___o "o"
#undef __GNUC__
#ifdef __GNUC__
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "gcc -fpic -O1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "gcc \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "gcc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "gcc -shared -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "gcc -fPIC -O1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#else
#define ___CC_O "cc -Wf,-XNg4000,-XNd16000,-XNp8000 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "cc -Wf,-XNg4000,-XNd16000,-XNp8000 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "cc \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "cc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "cc -shared -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "cc -Wf,-XNg4000,-XNd16000,-XNp8000 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#endif
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___GAMBCLIB_SH "libgambc.so"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "/usr/bin/ld -shared -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#endif

#ifdef __alpha
#define ___CPU_alpha
#define ___CPU_LITEND
#undef __GNUC__
#undef ___FORCE_32
#ifdef ___FORCE_32
#undef __GNUC__
#define ___o "o"
#define ___CC_O "cc -w -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "cc -w -fpic -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc -taso \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "cc -taso \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "cc -taso \$(OBJECTS) -lgambc -o"
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___GAMBCLIB_SH "libgambc.so"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "cc -taso -shared -expect_unresolved \\* -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "cc -w -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "cc -taso -shared -expect_unresolved \\* -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#else
#define ___o "o"
#ifdef __GNUC__
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "gcc -fpic -O1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "gcc \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "gcc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "gcc -shared -expect_unresolved \\* -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "gcc -fPIC -O1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#else
#define ___CC_O "cc -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "cc -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "cc \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "cc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "cc -shared -expect_unresolved \\* -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "cc -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#endif
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___GAMBCLIB_SH "libgambc.so"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "/usr/bin/ld -shared -expect_unresolved \\* -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#endif
#endif

#ifdef hpux
#ifdef __hp9000s300
#define ___CPU_m68k
#define ___CPU_BIGEND
#else
#define ___CPU_hppa
#define ___CPU_BIGEND
#endif
#define ___o "o"
#ifdef __GNUC__
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "gcc -fpic -O1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "gcc \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "gcc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "gcc -shared -fpic -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "gcc -fPIC -O1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#else
#define ___CC_O "cc +O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "cc +z +O1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "cc \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "cc \$(OBJECTS) -lgambc -o"
#define ___MAKE_GAMBCLIB_SH "cc -Wl,-b +z -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "cc +z -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#endif
#define ___CLIBS "-lm -ldld"
#define ___GAMBCLIB "libgambc.a"
#define ___GAMBCLIB_SH "libgambc.sl"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "/bin/ld -b -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___NO_DYNAMIC_IMPORTS
#define ___SETDLPATH "SHLIB_PATH=../lib"
#endif

#ifdef _AIX
#define ___CPU_rs6k
#define ___CPU_BIGEND
#define ___o "o"
#ifdef __GNUC__
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#else
#define ___CC_O "cc -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#endif
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "echo"
#define ___SETDLPATH "LIBPATH=../lib"
#endif

#ifdef linux
#ifdef i386
#define ___CPU_x86
#define ___CPU_LITEND
#define ___o "o"
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "gcc -fpic -O1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc -rdynamic \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "gcc -rdynamic \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "gcc -rdynamic \$(OBJECTS) -lgambc -o"
#define ___CLIBS "-lm -ldl"
#define ___GAMBCLIB "libgambc.a"
#define ___GAMBCLIB_SH "libgambc.so"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "gcc -shared -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "gcc -fPIC -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "/usr/bin/ld -shared -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#endif
#endif

#ifdef __FreeBSD__
#ifdef i386
#define ___CPU_x86
#define ___CPU_LITEND
#define ___PREPEND_UNDERSCORE
#define ___o "o"
#define ___CC_O "gcc -O1 -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "gcc -fpic -O1 -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "gcc \$(OBJECTS) -L../lib -lgambc -o"
#define ___LINK_SH_INSTALL "gcc \$(OBJECTS) -lgambc -o"
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___GAMBCLIB_SH "libgambc.so"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "gcc -shared -o \$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "gcc -fPIC -O1 -c -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -o "
#define ___CC_PROG2 ".o "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "/usr/bin/ld -Bshareable -o "
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".o"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#endif
#endif

#ifdef __DJGPP__
#ifdef __i386__
#define ___CPU_x86
#define ___CPU_LITEND
#define ___o "o"
#define ___CC_O "gcc -c \$(CC_O_ARGS)"
#define ___LINK "gcc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "echo"
#define ___SETDLPATH ""
#endif
#endif

#ifdef _MSC_VER
#if _MSC_VER < 1000
#ifdef _MI386
#define ___CPU_x86
#define ___CPU_LITEND
#define ___EXPORT __declspec(dllexport)
#define ___IMPORT __declspec(dllimport)
#define ___o "obj"
#define ___CC_O "cl -W0 -AH -Oitb1 -G3s \$(CFLAGS) -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "cl \$(LFLAGS) \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___CLIBS ""
#define ___GAMBCLIB "libgambc.lib"
#define ___GAMBCLIB_SH "gambc.dll"
#define ___MAKE_GAMBCLIB "lib -out:\$(GAMBCLIB) \$(OBJECTS)"
#define ___MAKE_GAMBCLIB_SH "link -dll \$(LFLAGS) -out:\$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "cl -W0 -AH -Oitb1 -G3s -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -c -Fo"
#define ___CC_PROG2 ".obj "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "link -nodefaultlib -noentry -dll -out:"
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".obj"
#define ___SETDLPATH "OLDPATH=\"\$\$PATH\" PATH=\"\$\$OLDPATH;..\\\\\\\\lib\""
#define ___NO_DYNAMIC_IMPORTS
#endif
#else
#ifdef _M_IX86
#define ___CPU_x86
#define ___CPU_LITEND
#define ___EXPORT __declspec(dllexport)
#define ___IMPORT __declspec(dllimport)
#define ___o "obj"
#define ___CC_O "cl -nologo -Oityb1 -Gs -MD \$(CFLAGS) -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___CC_O_SH "cl -nologo -Oityb1 -Gs -MD \$(CFLAGS) -c \$(CC_O_SH_ARGS) -D___SINGLE_HOST"
#define ___LINK "cl -nologo \$(LFLAGS) \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#define ___LINK_SH "cl -nologo \$(LFLAGS) \$(OBJECTS) ../lib/gambc.lib -o"
#define ___LINK_SH_INSTALL "cl -nologo \$(LFLAGS) \$(OBJECTS) ../lib/gambc.lib -o"
#define ___CLIBS "winmm.lib"
#define ___GAMBCLIB "libgambc.lib"
#define ___GAMBCLIB_SH "gambc.dll"
#define ___MAKE_GAMBCLIB "lib -out:\$(GAMBCLIB) \$(OBJECTS)"
#define ___MAKE_GAMBCLIB_SH "link -nologo -dll \$(LFLAGS) -out:\$(GAMBCLIB_SH) \$(OBJECTS) \$(CLIBS)"
#define ___CC_PROG1 "cl -nologo -Oityb1 -Gs -MD -D___SHARED -D___BIND_LATE -D___SINGLE_HOST -c -Fo"
#define ___CC_PROG2 ".obj "
#define ___CC_PROG3 ".c"
#define ___LD_PROG1 "link -nologo -nodefaultlib -noentry -dll -out:"
#define ___LD_PROG2 " "
#define ___LD_PROG3 ".obj"
#define ___SETDLPATH "OLDPATH=\"\$\$PATH\" PATH=\"\$\$OLDPATH;..\\\\\\\\lib\""
#define ___NO_DYNAMIC_IMPORTS
#endif
#endif
#endif

#ifdef __WATCOMC__
#ifdef __386__
#define ___CPU_x86
#define ___CPU_LITEND
#define ___EXPORT __export
#define ___o "obj"
#define ___CC_O "wcl386 -w0 -oetir -s -5r -fp5 -bt=windows -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "wlink option stack=16384 system dos4g file \$(OBJECTS_COMMA) library ..\\\\\\\\lib\\\\\\\\libgambc.lib name"
#define ___CLIBS ""
#define ___GAMBCLIB "libgambc.lib"
#define ___MAKE_GAMBCLIB "wlib -n -p=32 \$(GAMBCLIB) \$(OBJECTS_PLUS)"
#define ___MAKE_GAMBCLIB_SH "echo"
#define ___SETDLPATH "OLDPATH=\"\$\$PATH\" PATH=\"\$\$OLDPATH;..\\\\\\\\lib\""
#endif
#endif

#ifdef __MACHTEN__
#ifdef __MACHTEN_PPC__
#define ___CPU_ppc
#define ___CPU_BIGEND
#define ___LINK "cc -Xlstack=300000 \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#else
#define ___CPU_m68k
#define ___LINK "cc \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#endif
#define ___o "o"
#define ___CC_O "cc -c \$(CC_O_ARGS)"
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "echo"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#endif

#ifdef NeXT
#ifdef i386
#define ___CPU_x86
#define ___CPU_LITEND
#else
#define ___CPU_m68k
#define ___CPU_BIGEND
#endif
#define ___o "o"
#ifdef __GNUC__
#define ___CC_O "gcc -posix -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "gcc -posix \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#else
#define ___CC_O "cc -posix -c \$(CC_O_ARGS) -D___SINGLE_HOST"
#define ___LINK "cc -posix \$(OBJECTS) ../lib/\$(GAMBCLIB) \$(CLIBS) -o"
#endif
#define ___CLIBS "-lm"
#define ___GAMBCLIB "libgambc.a"
#define ___MAKE_GAMBCLIB "ar rc \$(GAMBCLIB) \$(OBJECTS);ranlib \$(GAMBCLIB)"
#define ___MAKE_GAMBCLIB_SH "echo"
#define ___SETDLPATH "LD_LIBRARY_PATH=../lib"
#endif

#ifdef __MWERKS__
#ifdef __POWERPC__
#define ___CPU_ppc
#define ___CPU_BIGEND
#else
#define ___CPU_m68k
#define ___CPU_BIGEND
#endif
#endif

/*---------------------------------------------------------------------------*/

/*
 * DETERMINE C COMPILER'S CHARACTERISTICS
 *
 * The following symbols need to be defined to reflect the
 * C compiler's characteristics:
 *
 *  define ___I16 as the integral type of exactly 16 bits
 *  define ___I32 as the integral type of exactly 32 bits
 *  define ___I64 as the integral type of exactly 64 bits (only if one exists)
 *  define ___F64 as the floating-point type of exactly 64 bits
 */

#ifdef ___CPU_alpha
#define ___I16 short
#define ___I32 int
#define ___I64 long
#define ___F64 double
#endif

#ifdef ___CPU_hppa
#define ___I16 short
#define ___I32 long
#define ___F64 double
#endif

#ifdef ___CPU_m68k
#define ___I16 short
#define ___I32 long
#define ___F64 double
#endif

#ifdef ___CPU_mips
#define ___I16 short
#define ___I32 long
#define ___F64 double
#endif

#ifdef ___CPU_sparc
#define ___I16 short
#define ___I32 long
#define ___F64 double
#endif

#ifdef ___CPU_ppc
#define ___I16 short
#define ___I32 long
#define ___F64 double
#endif

#ifdef ___CPU_rs6k
#define ___I16 short
#define ___I32 long
#define ___F64 double
#endif

#ifdef ___CPU_x86
#define ___I16 short
#define ___I32 long
#define ___F64 double
#endif

/*---------------------------------------------------------------------------*/

/* SANITY CHECKS AND SETTING OF DEFAULT OPTIONS */

/*
 * Verify that processor type is known.
 */

#ifndef ___I16
@error "___I16 must be defined"
#endif

#ifndef ___I32
@error "___I32 must be defined"
#endif

#ifndef ___F64
@error "___F64 must be defined"
#endif

#ifdef ___CPU_BIGEND
#ifdef ___CPU_LITEND
@error "Either ___CPU_BIGEND or ___CPU_LITEND must be defined"
#endif
#else
#ifndef ___CPU_LITEND
@error "Either ___CPU_BIGEND or ___CPU_LITEND must be defined"
#endif
#endif

/*
 * Generate a single host procedure or multiple C host
 * procedures (default) per Scheme module?
 */

#ifdef ___SINGLE_HOST
#ifdef ___MULTIPLE_HOSTS
@error "Define either ___SINGLE_HOST or ___MULTIPLE_HOSTS"
#endif
#else
#ifndef ___MULTIPLE_HOSTS
#define ___MULTIPLE_HOSTS
#endif
#endif

/*
 * Select binding time for global variables, symbols, and keywords.
 * Early binding (default) produces faster code because it
 * directly accesses the resource.  Late binding does an indirection
 * at run time.
 */

#ifdef ___BIND_EARLY
#ifdef ___BIND_LATE
@error "Define either ___BIND_EARLY or ___BIND_LATE"
#endif
#else
#ifndef ___BIND_LATE
#define ___BIND_EARLY
#endif
#endif

/*
 * Compiling to produce a library or an application with a "main" (default
 * unless compiling a flat link file).
 */

#ifdef ___LIBRARY
#ifdef ___APPLICATION
@error "Define either ___LIBRARY or ___APPLICATION"
#endif
#else
#ifndef ___APPLICATION
#ifdef ___FLAT_LINKFILE
#define ___LIBRARY
#else
#define ___APPLICATION
#endif
#endif
#endif

/*
 * Compiling to produce a shared-library or not (default).
 */

#ifdef ___SHARED
#ifdef ___NON_SHARED
@error "Define either ___SHARED or ___NON_SHARED"
#endif
#else
#ifndef ___NON_SHARED
#define ___NON_SHARED
#endif
#endif

/*
 * Size in bytes of characters in Scheme strings.
 *
 * ___CS must be 1, 2 or 4, which give respectively an 8 bit, 16 bit,
 * or 29 bit subset of Unicode (which is a 31 bit code).  The 8 bit
 * subset of Unicode corresponds to LATIN-1 and the 16 bit subset
 * of Unicode corresponds to UCS-2.
 */

#ifndef ___CS
#define ___CS 2
#endif

/*
 * Default character encoding to use for I/O.
 *
 * ___DEFAULT_IO_ENCODING should be one of:
 *    ___IO_CHAR_ENCODING    1 byte per character (native encoding)
 *    ___IO_LATIN1_ENCODING  1 byte per character (LATIN-1 encoding)
 *    ___IO_UCS4_ENCODING    4 bytes per character (UCS-4 encoding)
 *    ___IO_UCS2_ENCODING    2 bytes per character (UCS-2 encoding)
 *    ___IO_UTF8_ENCODING    1 to 6 bytes per character (UTF-8 encoding)
 */

#ifndef ___DEFAULT_IO_ENCODING
#define ___DEFAULT_IO_ENCODING ___IO_CHAR_ENCODING
#endif

/*
 * Number of registers in the virtual machine.  These definitions must
 * agree with those in the file "gsc/_t-c-1.scm".  All Scheme sources
 * must be recompiled if these definitions are changed.
 *
 * ___NB_GVM_REGS = total number of registers available
 *                  3 <= ___NB_GVM_REGS <= 8
 * ___NB_ARG_REGS = maximum number of arguments passed in registers
 *                  1 <= ___NB_ARG_REGS <= ___NB_GVM_REGS-2
 */

#define ___NB_GVM_REGS 5
#define ___NB_ARG_REGS 3

/*---------------------------------------------------------------------------*/

/*
 * IMPORTANT NOTE
 *
 * As a general principle, the macros are written in such a way that all
 * macros which expand into C expressions yield PRIMARY C expressions.
 * Also, macros assume that arguments are PRIMARY expressions.  A PRIMARY
 * expression is either:
 *
 *   an identifier, or
 *   a non-negative constant, or
 *   a parenthesized expression.
 *
 * This convention avoids many operator priority problems and helps keep
 * the number of parentheses down.  On the other hand, it is easy to
 * introduce errors when modifying these macros so be careful.
 */

/*---------------------------------------------------------------------------*/

/*
 * GENERAL DEFINITIONS
 *
 * ___WORD  must be an integral type able to contain a pointer with no loss
 * ___LWS   is equal to the base 2 logarithm of "sizeof(___WORD)"
 * ___U8    must be an unsigned integral type of exactly 8 bits
 * ___U16   must be an unsigned integral type of exactly 16 bits
 * ___U32   must be an unsigned integral type of exactly 32 bits
 */

#ifndef ___I64
#undef ___FORCE_32
#define ___FORCE_32
#endif

#ifdef ___FORCE_32
#ifdef ___I64
#define ___WORD_IS_SMALLER_THAN_POINTERS
#endif
#define ___WORD ___I32
#define ___LWS 2
#define ___FLONUM_SIZE 2
#else
#define ___WORD ___I64
#define ___LWS 3
#define ___FLONUM_SIZE 1
#endif

#define ___WS (1<<___LWS)
#define ___U8  unsigned char
#define ___U16 unsigned ___I16
#define ___U32 unsigned ___I32

#define ___LATIN1 ___U8
#define ___UCS4 ___U32
#define ___UCS2 ___U16
#define ___LATIN1STRING ___U8*
#define ___UCS4STRING ___UCS4*
#define ___UCS2STRING ___UCS2*
#define ___UTF8STRING char*

#if ___CS == 1

#define ___LCS 0
#define ___C ___U8

#else

#if ___CS == 2

#define ___LCS 1
#define ___C ___U16

#else

#define ___LCS 2
#define ___C ___U32

#endif
#endif

#ifdef ___WORD_IS_SMALLER_THAN_POINTERS
#define ___FAKEWORD ___WORD
#define ___CAST_FAKEWORD_TO_WORD(x)x
#define ___CAST_WORDSTAR_TO_FAKEWORD(x)((___WORD)x)
#define ___FAKEHOST ___WORD
#define ___CAST_HOST_TO_FAKEHOST(x)((___WORD)x)
#define ___CAST_FAKEHOST_TO_HOST(x)((___host)x)
#else
#define ___FAKEWORD ___WORD*
#define ___CAST_FAKEWORD_TO_WORD(x)((___WORD)x)
#define ___CAST_WORDSTAR_TO_FAKEWORD(x)x
#define ___FAKEHOST ___host
#define ___CAST_HOST_TO_FAKEHOST(x)x
#define ___CAST_FAKEHOST_TO_HOST(x)x
#endif

/* Padding at end of objects so that they can be aligned to an address */
/* multiple of 8 by moving them forward.                               */

#ifdef ___I64
#define ___PADDING
#define ___PADDING_LBL
#define ___PADDING_FLO
#else
#define ___PADDING ,0
#define ___PADDING_LBL ,{0,0,0,0}
#define ___PADDING_FLO ,0,0
#endif

/* Utility macro to select code for MULTIPLE/SINGLE host. */

#ifdef ___SINGLE_HOST
#define ___SM(s,m)s
#else
#define ___SM(s,m)m
#endif

/*---------------------------------------------------------------------------*/

/* OBJECT REPRESENTATION */

/*
 * For a complete description of the object representation, read the
 * file "lib/mem.c".
 */

/*
 * Type tag assignment
 *
 * Type tags are located in the lower 2 bits of a ___WORD.
 *
 * ___TB = number of tag bits
 * ___tFIXNUM   = tag for fixnums (small integers), must be 0
 * ___tSPECIAL  = tag for other immediates (#f, #t, (), chars, ...)
 * ___tPAIR     = tag for pairs
 * ___tSUBTYPED = tag for other memory allocated objects
 */

#define ___TB 2
#define ___tFIXNUM   0
#define ___tSPECIAL  2
#define ___tPAIR     3
#define ___tSUBTYPED 1

#define ___MEM_ALLOCATED(obj)(obj&1)

/*
 * Head tag assignment
 *
 * These tags are stored in the lower 2 bits of the head of memory
 * allocated objects.
 *
 * ___MOVABLE = tag for movable objects
 * ___FORW    = tag for movable objects that have been forwarded by GC
 * ___STILL   = tag for still objects
 * ___PERM    = tag for permanent objects
 */

#define ___MOVABLE 0
#define ___FORW    1
#define ___STILL   2
#define ___PERM    3

/*
 * Subtype tag assignment
 *
 * These tags are stored in the head of memory allocated objects
 * (including pairs).
 *
 * ___SB = number of subtype tag bits
 * ___sVECTOR       = tag for vectors
 * ___sSYMBOL       = tag for symbols
 * ___sKEYWORD      = tag for keywords
 * ___sPORT         = tag for ports
 * ___sRATNUM       = tag for ratnums
 * ___sCPXNUM       = tag for cpxnums
 * ___sFRAME        = tag for continuation frames
 * ___sSTRUCTURE    = tag for structures
 * ___sPOINTER      = tag for C pointers
 * ___sCONTINUATION = tag for continuation descriptors
 * ___sPLACEHOLDER  = tag for placeholders
 * ___sPROCEDURE    = tag for procedures
 * ___sPAIR         = tag for pairs
 * ___sMEROON       = tag for Meroon objects
 * ___sWILL         = tag for wills
 * ___sSTRING       = tag for strings
 * ___sBIGNUM       = tag for bignums
 * ___sFLONUM       = tag for flonums
 * ___sVECTOR8      = tag for 8-bit integer vectors
 * ___sVECTOR16     = tag for 16-bit integer vectors
 */

#define ___SB 5
#define ___sVECTOR       0
#define ___sSYMBOL       1
#define ___sKEYWORD      13
#define ___sPORT         2
#define ___sRATNUM       3
#define ___sCPXNUM       4
#define ___sFRAME        5
#define ___sSTRUCTURE    6
#define ___sPOINTER      7
#define ___sCONTINUATION 8
#define ___sPLACEHOLDER  9
#define ___sPROCEDURE    10
#define ___sPAIR         11
#define ___sMEROON       12
#define ___sWILL         15
#define ___sSTRING       16
#define ___sBIGNUM       17
#define ___sFLONUM       18
#define ___sVECTOR8      19
#define ___sVECTOR16     20

/*
 * Miscellaneous masks.
 */

#define ___RF ___TB
#define ___SF (___RF+1)
#define ___LF (___SF+___SB)

#define ___TMASK ((1<<___TB)-1)
#define ___RMASK (1<<___TB)
#define ___SMASK (((1<<___SB)-1)<<___SF)
#define ___LMASK (~(___U32)0<<___LF)

/*
 * Value constructors for object references inside executable code.
 */

#define ___FAL (((___WORD)-1<<___TB)+___tSPECIAL)
#define ___TRU (((___WORD)-2<<___TB)+___tSPECIAL)
#define ___NUL (((___WORD)-3<<___TB)+___tSPECIAL)
#define ___EOF (((___WORD)-4<<___TB)+___tSPECIAL)
#define ___VOID (((___WORD)-5<<___TB)+___tSPECIAL)
#define ___ABSENT (((___WORD)-6<<___TB)+___tSPECIAL)
#define ___UNB1 (((___WORD)-7<<___TB)+___tSPECIAL)
#define ___UNB2 (((___WORD)-8<<___TB)+___tSPECIAL)
#define ___BRK (((___WORD)-9<<___TB)+___tSPECIAL)
#define ___SCRIPT (((___WORD)-10<<___TB)+___tSPECIAL)
#define ___OPTIONAL (((___WORD)-11<<___TB)+___tSPECIAL)
#define ___KEY_OBJ (((___WORD)-12<<___TB)+___tSPECIAL)
#define ___REST (((___WORD)-13<<___TB)+___tSPECIAL)
#define ___FIX(x)((___WORD)((___I32)x<<___TB))
#define ___CHR(x)(((___WORD)(___C)x<<___TB)+___tSPECIAL)

#define ___SYM(i,id)___CAST_FAKEWORD_TO_WORD(___sym_tbl[i])
#define ___KEY(i,id)___CAST_FAKEWORD_TO_WORD(___key_tbl[i])
#define ___CNS(i)___TAG((___ALIGNUP(___cns_tbl,___WS)+i*(___PAIR_SIZE+1)),___tPAIR)
#define ___SUB(i)___CAST_FAKEWORD_TO_WORD(___sub_tbl[i])

/*
 * Value constructors for object references inside constant
 * memory allocated objects.
 */

#define ___REF_FAL ___FAL
#define ___REF_TRU ___TRU
#define ___REF_NUL ___NUL
#define ___REF_EOF ___EOF
#define ___REF_VOID ___VOID
#define ___REF_ABSENT ___ABSENT
#define ___REF_SCRIPT ___SCRIPT
#define ___REF_OPTIONAL ___OPTIONAL
#define ___REF_KEY_OBJ ___KEY_OBJ
#define ___REF_REST ___REST
#define ___REF_FIX(x)___FIX(x)
#define ___REF_CHR(x)___CHR(x)

#define ___REF_SYM(i,id)(((___WORD)(-1-i)<<___TB)+___tPAIR)
#define ___REF_KEY(i,id)(((___WORD)(-1-i)<<___TB)+___tSUBTYPED)
#define ___REF_CNS(i)(((___WORD)i<<___TB)+___tPAIR)
#define ___REF_SUB(i)(((___WORD)i<<___TB)+___tSUBTYPED)

/*---------------------------------------------------------------------------*/

/* Miscellaneous macros */

#define ___NOTHING

#define ___INT(x)(x>>___TB)

#define ___RADIX_WIDTH 14

#define ___MIN_FIX (-((___I32)1<<(32-___TB-1)))
#define ___MAX_FIX (((___I32)1<<(32-___TB-1))-1)

#if ___CS == 4
#define ___MAX_CHR ___MAX_FIX
#else
#define ___MAX_CHR (((___I32)1<<(8*___CS))-1)
#endif

/*---------------------------------------------------------------------------*/

#define ___ALIGNUP(x,mult)((___WORD*)(((___WORD)x+mult-1)&(-mult)))

/* Type tests */

#define ___TAG(ptr,tag)((___WORD)ptr+tag)
#define ___UNTAG(obj)((___WORD*)(obj&-(1<<___TB)))
#define ___UNTAG_AS(obj,tag)((___WORD*)(obj-tag))

#define ___TYP(x)(x&___TMASK)
#define ___TESTTYPE(x,typ)(___TYP(x)==typ)

#define ___TESTSUBTYPE(x,s)(___TYP((___temp=x))==___tSUBTYPED&&___SUBTYPE(___temp)==s<<___TB)
#define ___HEADER(x)(*___UNTAG_AS(x,___tSUBTYPED))

#define ___BODY_OFS 1

#ifdef ___USE_HANDLES
#define ___BODY(obj)((___WORD*)(*(___UNTAG(obj)+___BODY_OFS)+___BODY_OFS))
#define ___BODY_AS(obj,tag)((___WORD*)*(___UNTAG_AS(obj,tag)+___BODY_OFS)+___BODY_OFS)
#else
#define ___BODY(obj)(___UNTAG(obj)+___BODY_OFS)
#define ___BODY_AS(obj,tag)(___UNTAG_AS(obj,tag)+___BODY_OFS)
#endif

#define ___MAKE_HD(bytes,subtype,refcount,tag) \
((bytes<<___LF)+(subtype<<___SF)+refcount+tag)
#define ___HD_BYTES(head)((___U32)head>>___LF)
#define ___HD_WORDS(head)((((___U32)head+((___WS-1)<<___LF)))>>(___LF+___LWS))
#define ___HD_FIELDS(head)((___U32)head>>(___LF+___LWS))
#define ___HD_SUBTYPE(head)((___U32)head>>___SF&((1<<___SB)-1))
#define ___HD_REFCOUNT(head)((___U32)head&___RMASK)

#define ___FIELD(obj,i)(*(___BODY_AS(obj,___tSUBTYPED)+i))

#define ___RC0 0
#define ___RCN ___RMASK
#define ___RESET_RC(head)(head&~(___WORD)___RMASK)
#define ___ADD_RC(head,refcount)(head|refcount)
#define ___UNREFERENCED(head)((head&___RMASK)==0)

#define ___WORDS(bytes)((bytes+___WS-1)>>___LWS)

#define ___MAKE_HD_BYTES(bytes,subtype)___MAKE_HD(bytes,subtype,___RCN,___MOVABLE)
#define ___MAKE_HD_WORDS(words,subtype)___MAKE_HD((words<<___LWS),subtype,___RCN,___MOVABLE)

#define ___SYMBOL_SIZE 3
#define ___KEYWORD_SIZE 2
#define ___WILL_SIZE 3
#define ___PLACEHOLDER_SIZE 2
#define ___RATNUM_SIZE 2
#define ___CPXNUM_SIZE 2
#define ___CONTINUATION_SIZE 3
#define ___INTRO_SIZE 3

/* Flonum boxing and unboxing */

#define ___FLONUM_VAL(obj)(*(___F64*)(___BODY_AS(obj,___tSUBTYPED)))

#define ___FLONUM_SETUP(ptr,x) \
(*ptr=___MAKE_HD_WORDS(___FLONUM_SIZE,___sFLONUM),*(___F64*)(ptr+1)=x, \
___TAG(ptr,___tSUBTYPED))

#if ___WS == 4

#define ___FLOBOX(x) \
(___hp+=___FLONUM_SIZE+2,((___WORD)___hp&7)==0 \
?___FLONUM_SETUP((___hp-___FLONUM_SIZE-1),x) \
:___FLONUM_SETUP((___hp-___FLONUM_SIZE-2),x))

#else

#define ___FLOBOX(x) \
(___hp+=___FLONUM_SIZE+1,___FLONUM_SETUP((___hp-___FLONUM_SIZE-1),x))

#endif

#define ___FLOUNBOX(x)___FLONUM_VAL(x)

#define ___BEGIN_FR {
#define ___END_FR }
#define ___D_FR(x)___F64 x;
#define ___SET_FR(x,y)x=y;

#define ___ROUND ___round

/*****************************************************************************/

/* Inlinable operations (for APPLY and IFJUMP gvm instructions) */

/* APPLY-able operations */

#define ___TYPE(x)___FIX(___TYP(x))
#define ___TYPECAST(x,y)((x&~___TMASK)+___INT(y))
#define ___SUBTYPE(x)((___HEADER(x)&___SMASK)>>(___SF-___TB))
#define ___SUBTYPESET(x,y)___temp=x;___HEADER(___temp)=(___HEADER(___temp)&~___SMASK)+((y)<<(___SF-___TB));

/* IFJUMP-able operations */

#define ___FALSEP(x)(x==___FAL)
#define ___NULLP(x)(x==___NUL)
#define ___UNBOUNDP(x)(((___temp=x)==___UNB1)||(___temp==___UNB2))
#define ___EQP(x,y)(x==y)
#define ___EOFP(x)(x==___EOF)

/* IFJUMP-able operations */

#define ___FIXNUMP(x)___TESTTYPE(x,___tFIXNUM)
#define ___FLONUMP(x)___TESTSUBTYPE(x,___sFLONUM)
#define ___SPECIALP(x)___TESTTYPE(x,___tSPECIAL)
#define ___PAIRP(x)___TESTTYPE(x,___tPAIR)
#define ___SUBTYPEDP(x)___TESTTYPE(x,___tSUBTYPED)
#define ___PROCEDUREP(x)___TESTSUBTYPE(x,___sPROCEDURE)
#define ___PLACEHOLDERP(x)___TESTSUBTYPE(x,___sPLACEHOLDER)
#define ___VECTORP(x)___TESTSUBTYPE(x,___sVECTOR)
#define ___SYMBOLP(x)___TESTSUBTYPE(x,___sSYMBOL)
#define ___KEYWORDP(x)___TESTSUBTYPE(x,___sKEYWORD)
#define ___PORTP(x)___TESTSUBTYPE(x,___sPORT)
#define ___RATNUMP(x)___TESTSUBTYPE(x,___sRATNUM)
#define ___CPXNUMP(x)___TESTSUBTYPE(x,___sCPXNUM)
#define ___STRINGP(x)___TESTSUBTYPE(x,___sSTRING)
#define ___STRUCTUREP(x)___TESTSUBTYPE(x,___sSTRUCTURE)
#define ___BIGNUMP(x)___TESTSUBTYPE(x,___sBIGNUM)
#define ___CHARP(x)(___TYP((___temp=x))==___tSPECIAL&&___temp>=0)

/* APPLY-able operations */

#define ___FIX0 ___FIX(0)
#define ___FIXPOS(x)((___WORD)((___I32)x))
#define ___FIXADD(x,y)((___WORD)((___I32)x+(___I32)y))
#define ___FIX1 ___FIX(1)
#define ___FIXMUL(x,y)((___WORD)((___I32)x*(___I32)___INT(y)))
#define ___FIXNEG(x)((___WORD)(-(___I32)x))
#define ___FIXSUB(x,y)((___WORD)((___I32)x-(___I32)y))
#define ___FIXINV(x)___FIX(((___I32)___FIX(1)/(___I32)x))
#define ___FIXQUO(x,y)___FIX(((___I32)x/(___I32)y))
#define ___FIXREM(x,y)((___WORD)((___I32)x%(___I32)y))/****is this ok?*******/
#define ___FIXMOD(x,y)((___temp=___FIXREM((___ps->temp1=x),(___ps->temp2=y)))==0?___FIX(0):((___ps->temp1<0)==(___ps->temp2<0)?(___I32)___temp:___FIXADD(___temp,___ps->temp2)))
#define ___FIXIOR(x,y)((___WORD)((___I32)x|(___I32)y))
#define ___FIXXOR(x,y)((___WORD)((___I32)x^(___I32)y))
#define ___FIXM1 ___FIX(-1)
#define ___FIXAND(x,y)((___WORD)((___I32)x&(___I32)y))
#define ___FIXNOT(x)((___WORD)((___I32)x^(___I32)~___TMASK))
#define ___FIXASHR(x,y)(((___I32)x>>___INT(y))&~___TMASK)
#define ___FIXLSHR(x,y)((___I32)(((___U32)x>>___INT(y))&~___TMASK))
#define ___FIXSHL(x,y)((___I32)x<<___INT(y))
#define ___FIXTOCHR(x)((x&~___TMASK)+___tSPECIAL)
#define ___FIXFROMCHR(x)(x&~___TMASK)

/* IFJUMP-able operations */

#define ___FIXZEROP(x)(x==0)
#define ___FIXPOSITIVEP(x)(x>0)
#define ___FIXNEGATIVEP(x)(x<0)
#define ___FIXODDP(x)(x&___FIX(1))
#define ___FIXEVENP(x)!(x&___FIX(1))
#define ___FIXEQ(x,y)(x==y)
#define ___FIXLT(x,y)(x<y)
#define ___FIXGT(x,y)(x>y)
#define ___FIXLE(x,y)(x<=y)
#define ___FIXGE(x,y)(x>=y)

/* APPLY-able operations */

#define ___FLOTOFIX(x)___FIX(___FLOUNBOX(x))
#define ___FLOFROMFIX(x)___INT(x)
#define ___FLO0 0.0
#define ___FLOPOS(x)x
#define ___FLOADD(x,y)(x+y)
#define ___FLO1 1.0
#define ___FLOMUL(x,y)(x*y)
#define ___FLONEG(x)(-x)
#define ___FLOSUB(x,y)(x-y)
#define ___FLOINV(x)1.0/x
#define ___FLODIV(x,y)(x/y)
#define ___FLOABS(x)___EXT(fabs)(x)
#define ___FLOFLOOR(x)___EXT(floor)(x)
#define ___FLOCEILING(x)___EXT(ceil)(x)
#define ___FLOROUND(x)___EXT(___ROUND)(x)
#define ___FLOEXP(x)___EXT(exp)(x)
#define ___FLOLOG(x)___EXT(log)(x)
#define ___FLOSIN(x)___EXT(sin)(x)
#define ___FLOCOS(x)___EXT(cos)(x)
#define ___FLOTAN(x)___EXT(tan)(x)
#define ___FLOASIN(x)___EXT(asin)(x)
#define ___FLOACOS(x)___EXT(acos)(x)
#define ___FLOATAN(x)___EXT(atan)(x)
#define ___FLOSQRT(x)___EXT(sqrt)(x)

/* IFJUMP-able operations */

#define ___FLOZEROP(x)(x==0.0)
#define ___FLOPOSITIVEP(x)(x>0.0)
#define ___FLONEGATIVEP(x)(x<0.0)
#define ___FLOEQ(x,y)(x==y)
#define ___FLOLT(x,y)(x<y)
#define ___FLOGT(x,y)(x>y)
#define ___FLOLE(x,y)(x<=y)
#define ___FLOGE(x,y)(x>=y)

/* IFJUMP-able operations */

#define ___CHAREQP(x,y)(x==y)
#define ___CHARLTP(x,y)(x<y)
#define ___CHARGTP(x,y)(x>y)
#define ___CHARLEP(x,y)(x<=y)
#define ___CHARGEP(x,y)(x>=y)

/* APPLY-able operations */

#define ___PAIR_SIZE 2
#define ___PAIR_CDR_OFS 0
#define ___PAIR_CAR_OFS 1

#define ___PAIR_CAR(obj)(*(___BODY_AS(obj,___tPAIR)+___PAIR_CAR_OFS))
#define ___PAIR_CDR(obj)(*(___BODY_AS(obj,___tPAIR)+___PAIR_CDR_OFS))

#define ___CONS(x,y)(___hp[0]=___MAKE_HD_WORDS(___PAIR_SIZE,___sPAIR), \
___hp[___PAIR_CAR_OFS+1]=x,___hp[___PAIR_CDR_OFS+1]=y,___hp+=___PAIR_SIZE+1,___TAG((___hp-___PAIR_SIZE-1),___tPAIR))

#define ___SETCAR(obj,car)___PAIR_CAR(obj)=car;
#define ___SETCDR(obj,cdr)___PAIR_CDR(obj)=cdr;
#define ___CAR(obj)___PAIR_CAR(obj)
#define ___CDR(obj)___PAIR_CDR(obj)
#define ___CAAR(obj)___CAR(___CAR(obj))
#define ___CADR(obj)___CAR(___CDR(obj))
#define ___CDAR(obj)___CDR(___CAR(obj))
#define ___CDDR(obj)___CDR(___CDR(obj))
#define ___CAAAR(obj)___CAR(___CAR(___CAR(obj)))
#define ___CAADR(obj)___CAR(___CAR(___CDR(obj)))
#define ___CADAR(obj)___CAR(___CDR(___CAR(obj)))
#define ___CADDR(obj)___CAR(___CDR(___CDR(obj)))
#define ___CDAAR(obj)___CDR(___CAR(___CAR(obj)))
#define ___CDADR(obj)___CDR(___CAR(___CDR(obj)))
#define ___CDDAR(obj)___CDR(___CDR(___CAR(obj)))
#define ___CDDDR(obj)___CDR(___CDR(___CDR(obj)))
#define ___CAAAAR(obj)___CAR(___CAR(___CAR(___CAR(obj))))
#define ___CAAADR(obj)___CAR(___CAR(___CAR(___CDR(obj))))
#define ___CAADAR(obj)___CAR(___CAR(___CDR(___CAR(obj))))
#define ___CAADDR(obj)___CAR(___CAR(___CDR(___CDR(obj))))
#define ___CADAAR(obj)___CAR(___CDR(___CAR(___CAR(obj))))
#define ___CADADR(obj)___CAR(___CDR(___CAR(___CDR(obj))))
#define ___CADDAR(obj)___CAR(___CDR(___CDR(___CAR(obj))))
#define ___CADDDR(obj)___CAR(___CDR(___CDR(___CDR(obj))))
#define ___CDAAAR(obj)___CDR(___CAR(___CAR(___CAR(obj))))
#define ___CDAADR(obj)___CDR(___CAR(___CAR(___CDR(obj))))
#define ___CDADAR(obj)___CDR(___CAR(___CDR(___CAR(obj))))
#define ___CDADDR(obj)___CDR(___CAR(___CDR(___CDR(obj))))
#define ___CDDAAR(obj)___CDR(___CDR(___CAR(___CAR(obj))))
#define ___CDDADR(obj)___CDR(___CDR(___CAR(___CDR(obj))))
#define ___CDDDAR(obj)___CDR(___CDR(___CDR(___CAR(obj))))
#define ___CDDDDR(obj)___CDR(___CDR(___CDR(___CDR(obj))))

/* APPLY-able operations */

#define ___MAKECELL(x)(___hp+=2,___hp[-2]=___MAKE_HD_WORDS(1,___sVECTOR), \
___hp[-1]=x,___TAG((___hp-2),___tSUBTYPED))
#define ___CELLREF(x)___BODY_AS(x,___tSUBTYPED)[0]
#define ___CELLSET(x,y)___BODY_AS(x,___tSUBTYPED)[0]=y;

#define ___VECTORLENGTH(x)___FIX(___HD_FIELDS(___HEADER(x)))
#define ___VECTORREF(x,y)___BODY_AS(x,___tSUBTYPED)[___INT(y)]
#define ___VECTORSET(x,y,z)___BODY_AS(x,___tSUBTYPED)[___INT(y)]=z;
#define ___VECTORSHRINK(x,y)___temp=x;___HEADER(___temp)=(___HEADER(___temp)&~___LMASK)+((y)<<(___LF-___TB));

#define ___STRINGLENGTH(x)___FIX((___HD_BYTES(___HEADER(x))>>___LCS))
#define ___STRINGREF(x,y)___CHR(((___C*)___BODY_AS(x,___tSUBTYPED))[___INT(y)])
#define ___STRINGSET(x,y,z)((___C*)___BODY_AS(x,___tSUBTYPED))[___INT(y)]=___INT(z);
#define ___STRINGSHRINK(x,y)___temp=x;___HEADER(___temp)=(___HEADER(___temp)&~___LMASK)+((y)<<(___LF-___TB+___LCS));

#define ___VECTOR8LENGTH(x)___FIX(___HD_BYTES(___HEADER(x)))
#define ___VECTOR8REF(x,y)___FIX(((___U8*)___BODY_AS(x,___tSUBTYPED))[___INT(y)])
#define ___VECTOR8SET(x,y,z)((___U8*)___BODY_AS(x,___tSUBTYPED))[___INT(y)]=___INT(z);
#define ___VECTOR8SHRINK(x,y)___temp=x;___HEADER(___temp)=(___HEADER(___temp)&~___LMASK)+((y)<<(___LF-___TB));

#define ___VECTOR16LENGTH(x)___FIX((___HD_BYTES(___HEADER(x))>>1))
#define ___VECTOR16REF(x,y)___FIX(((___U16*)___BODY_AS(x,___tSUBTYPED))[___INT(y)])
#define ___VECTOR16SET(x,y,z)((___U16*)___BODY_AS(x,___tSUBTYPED))[___INT(y)]=___INT(z);
#define ___VECTOR16SHRINK(x,y)___temp=x;___HEADER(___temp)=(___HEADER(___temp)&~___LMASK)+((y)<<(___LF-___TB+1));

#define ___STRUCTREF(x,y,tag)___VECTORREF(x,y)
#define ___STRUCTSET(x,y,z,tag)___VECTORSET(x,y,z)

#define ___SYMBOLTOSTR(x)___VECTORREF(x,___FIX(0))

#define ___KEYWORDTOSTR(x)___VECTORREF(x,___FIX(0))

#define ___CLOSURECODE(x)___CLO(x,0)
#define ___CLOSUREREF(x,y)___CLO(x,___INT(y))
#define ___CLOSURESET(x,y,z)___SET_CLO(x,___INT(y),z)

#define ___GLOBALVARREF(gv)((___glo_struct*)___VECTORREF(gv,___FIX(2)))->val
#define ___GLOBALVARSET(gv,x)((___glo_struct*)___VECTORREF(gv,___FIX(2)))->val = x;

#define ___MAKEPLACEHOLDER(x) \
(___hp+=___PLACEHOLDER_SIZE+1, \
___hp[-3]=___MAKE_HD_WORDS(___PLACEHOLDER_SIZE,___sPLACEHOLDER), \
___hp[-2]=x, ___hp[-1]=___TAG((___hp-___PLACEHOLDER_SIZE-1),___tSUBTYPED))

#define ___WILLP(x)___TESTSUBTYPE(x,___sWILL)
#define ___MAKEWILL(owner,action) \
(___hp+=___WILL_SIZE+1, \
___hp[-4]=___MAKE_HD_WORDS(___WILL_SIZE,___sWILL), \
___hp[-3]=owner, \
___hp[-2]=action, \
___hp[-1]=___ps->non_executable_wills, \
___ps->non_executable_wills=___TAG((___hp-___WILL_SIZE-1),___tSUBTYPED))
#define ___WILLOWNER(x) \
(___temp=___BODY_AS(x,___tSUBTYPED)[0], \
___MEM_ALLOCATED(___temp)?___BODY(___temp)[-1]=___ADD_RC(___BODY(___temp)[-1],___RCN):0, \
___temp)

/*---------------------------------------------------------------------------*/

/* Stack manipulation */

#define ___PUSH(val)*--___fp=(val);
#define ___POP *___fp++
#define ___PSSTK(fpo)___PSFP[-(fpo)]
#define ___PSADJFP(fpa)___PSFP-=(fpa);
#define ___STK(fpo)___fp[-(fpo)]
#define ___SET_STK(fpo,val)___fp[-(fpo)]=(val);
#define ___ADJFP(fpa)___fp-=(fpa);

#define ___PRM(i,glo)___GLOSTRUCT(i,glo).prm
#define ___GLO(i,glo)___GLOSTRUCT(i,glo).val
#define ___SET_GLO(i,glo,x)___GLOSTRUCT(i,glo).val=x;
#ifdef ___BIND_LATE
#define ___GLOSTRUCT(i,glo)(*___glo_tbl[i])
#else
#define ___GLOSTRUCT(i,glo)glo
#endif

#define ___HOST_PROC ___SM(___MH_PROC,___PH_PROC)
#define ___HOST_LBL0 ___SM(1,___PH_LBL0)

#define ___BEGIN_M_COD ___SM(___BEGIN_COD,___NOTHING)
#define ___END_M_COD ___SM(___END_COD,___NOTHING)
#define ___BEGIN_P_COD ___SM(___NOTHING,___BEGIN_COD)
#define ___END_P_COD ___SM(___NOTHING,___END_COD)

#define ___BEGIN_P_SLBL
#define ___DEF_P_SLBL(id)
#define ___END_P_SLBL

#define ___D_ALL ___SM(___MD_ALL,___PD_ALL)
#define ___R_ALL ___SM(___MR_ALL,___PR_ALL)
#define ___W_ALL ___SM(___MW_ALL,___PW_ALL)

#define ___BEGIN_COD \
___HIDDEN ___WORD ___HOST_PROC (___ps)___processor ___ps;{ \
register ___WORD ___pc, ___temp; \
register ___WORD ___start=___lp+(___HOST_LBL0*___LS*___WS); \
___D_ALL ___R_ALL

#define ___END_COD \
___W_ALL return ___pc;}

#define ___LS 4

#define ___PSHEAP ___ps->hp
#define ___D_HEAP register ___WORD *___hp;
#define ___R_HEAP ___hp=___PSHEAP;
#define ___W_HEAP ___PSHEAP=___hp;

#define ___PSFP ___ps->fp
#define ___D_FP register ___WORD *___fp;
#define ___R_FP ___fp=___PSFP;
#define ___W_FP ___PSFP=___fp;

/*---------------------------------------------------------------------------*/

/* GVM registers */

#define ___PSR0 ___ps->r[0]
#define ___D_R0 register ___WORD ___r0;
#define ___R_R0 ___r0=___PSR0;
#define ___W_R0 ___PSR0=___r0;
#define ___R0 ___r0
#define ___SET_R0(val)___r0=val;
#define ___PUSH_R0 ___PUSH(___R0)
#define ___POP_R0 ___SET_R0(___POP)

#if ___NB_ARG_REGS == 0
#define ___PUSH_ARGS_IN_REGS(na)
#define ___POP_ARGS_IN_REGS(na)
#define ___SET_SELF(val)___SET_R1(val)
#define ___SELF ___R1
#define ___LD_ARG_REGS
#define ___ST_ARG_REGS
#endif

#define ___PSR1 ___ps->r[1]
#define ___D_R1 register ___WORD ___r1;
#define ___R_R1 ___r1=___PSR1;
#define ___W_R1 ___PSR1=___r1;
#define ___R1 ___r1
#define ___SET_R1(val)___r1=(val);
#define ___PUSH_R1_TO_R1 ___PUSH(___R1)
#define ___POP_R1_TO_R1 ___SET_R1(___POP)
#define ___LD_R1_TO_R1 ___D_R1 ___R_R1
#define ___ST_R1_TO_R1 ___W_R1

#if ___NB_ARG_REGS == 1
#define ___PUSH_ARGS_IN_REGS(na) \
if ((na)>0) ___PUSH_R1_TO_R1
#define ___POP_ARGS_IN_REGS(na) \
if ((na)>0) ___POP_R1_TO_R1
#define ___SET_SELF(val)___SET_R2(val)
#define ___SELF ___R2
#define ___LD_ARG_REGS ___LD_R1_TO_R1
#define ___ST_ARG_REGS ___ST_R1_TO_R1
#endif

#if ___NB_GVM_REGS == 2

#define ___PUSH_REGS ___PUSH_R0 ___PUSH_R1_TO_R1
#define ___POP_REGS ___POP_R1_TO_R1 ___POP_R0

#else

#define ___PSR2 ___ps->r[2]
#define ___D_R2 register ___WORD ___r2;
#define ___R_R2 ___r2=___PSR2;
#define ___W_R2 ___PSR2=___r2;
#define ___R2 ___r2
#define ___SET_R2(val)___r2=(val);
#define ___PUSH_R1_TO_R2 ___PUSH_R1_TO_R1 ___PUSH(___R2)
#define ___POP_R2_TO_R1 ___SET_R2(___POP)___POP_R1_TO_R1
#define ___LD_R1_TO_R2 ___D_R2 ___LD_R1_TO_R1 ___R_R2
#define ___ST_R1_TO_R2 ___ST_R1_TO_R1 ___W_R2

#if ___NB_ARG_REGS == 2
#define ___PUSH_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___PUSH_R1_TO_R1 break; \
  default: ___PUSH_R1_TO_R2 \
  }
#define ___POP_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___POP_R1_TO_R1 break; \
  default: ___POP_R2_TO_R1 \
  }
#define ___SET_SELF(val)___SET_R3(val)
#define ___SELF ___R3
#define ___LD_ARG_REGS ___LD_R1_TO_R2
#define ___ST_ARG_REGS ___ST_R1_TO_R2
#endif

#if ___NB_GVM_REGS == 3

#define ___PUSH_REGS ___PUSH_R0 ___PUSH_R1_TO_R2
#define ___POP_REGS ___POP_R2_TO_R1 ___POP_R0

#else

#define ___PSR3 ___ps->r[3]
#define ___D_R3 register ___WORD ___r3;
#define ___R_R3 ___r3=___PSR3;
#define ___W_R3 ___PSR3=___r3;
#define ___R3 ___r3
#define ___SET_R3(val)___r3=(val);
#define ___PUSH_R1_TO_R3 ___PUSH_R1_TO_R2 ___PUSH(___R3)
#define ___POP_R3_TO_R1 ___SET_R3(___POP)___POP_R2_TO_R1
#define ___LD_R1_TO_R3 ___D_R3 ___LD_R1_TO_R2 ___R_R3
#define ___ST_R1_TO_R3 ___ST_R1_TO_R2 ___W_R3

#if ___NB_ARG_REGS == 3
#define ___PUSH_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___PUSH_R1_TO_R1 break; \
  case 2:  ___PUSH_R1_TO_R2 break; \
  default: ___PUSH_R1_TO_R3 \
  }
#define ___POP_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___POP_R1_TO_R1 break; \
  case 2:  ___POP_R2_TO_R1 break; \
  default: ___POP_R3_TO_R1 \
  }
#define ___SET_SELF(val)___SET_R4(val)
#define ___SELF ___R4
#define ___LD_ARG_REGS ___LD_R1_TO_R3
#define ___ST_ARG_REGS ___ST_R1_TO_R3
#endif

#if ___NB_GVM_REGS == 4

#define ___PUSH_REGS ___PUSH_R0 ___PUSH_R1_TO_R3
#define ___POP_REGS ___POP_R3_TO_R1 ___POP_R0

#else

#define ___PSR4 ___ps->r[4]
#define ___D_R4 register ___WORD ___r4;
#define ___R_R4 ___r4=___PSR4;
#define ___W_R4 ___PSR4=___r4;
#define ___R4 ___r4
#define ___SET_R4(val)___r4=(val);
#define ___PUSH_R1_TO_R4 ___PUSH_R1_TO_R3 ___PUSH(___R4)
#define ___POP_R4_TO_R1 ___SET_R4(___POP)___POP_R3_TO_R1
#define ___LD_R1_TO_R4 ___D_R4 ___LD_R1_TO_R3 ___R_R4
#define ___ST_R1_TO_R4 ___ST_R1_TO_R3 ___W_R4

#if ___NB_ARG_REGS == 4
#define ___PUSH_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___PUSH_R1_TO_R1 break; \
  case 2:  ___PUSH_R1_TO_R2 break; \
  case 3:  ___PUSH_R1_TO_R3 break; \
  default: ___PUSH_R1_TO_R4 \
  }
#define ___POP_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___POP_R1_TO_R1 break; \
  case 2:  ___POP_R2_TO_R1 break; \
  case 3:  ___POP_R3_TO_R1 break; \
  default: ___POP_R4_TO_R1 \
  }
#define ___SET_SELF(val)___SET_R5(val)
#define ___SELF ___R5
#define ___LD_ARG_REGS ___LD_R1_TO_R4
#define ___ST_ARG_REGS ___ST_R1_TO_R4
#endif

#if ___NB_GVM_REGS == 5

#define ___PUSH_REGS ___PUSH_R0 ___PUSH_R1_TO_R4
#define ___POP_REGS ___POP_R4_TO_R1 ___POP_R0

#else

#define ___PSR5 ___ps->r[5]
#define ___D_R5 register ___WORD ___r5;
#define ___R_R5 ___r5=___PSR5;
#define ___W_R5 ___PSR5=___r5;
#define ___R5 ___r5
#define ___SET_R5(val)___r5=(val);
#define ___PUSH_R1_TO_R5 ___PUSH_R1_TO_R4 ___PUSH(___R5)
#define ___POP_R5_TO_R1 ___SET_R5(___POP)___POP_R4_TO_R1
#define ___LD_R1_TO_R5 ___D_R5 ___LD_R1_TO_R4 ___R_R5
#define ___ST_R1_TO_R5 ___ST_R1_TO_R4 ___W_R5

#if ___NB_ARG_REGS == 5
#define ___PUSH_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___PUSH_R1_TO_R1 break; \
  case 2:  ___PUSH_R1_TO_R2 break; \
  case 3:  ___PUSH_R1_TO_R3 break; \
  case 4:  ___PUSH_R1_TO_R4 break; \
  default: ___PUSH_R1_TO_R5 \
  }
#define ___POP_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___POP_R1_TO_R1 break; \
  case 2:  ___POP_R2_TO_R1 break; \
  case 3:  ___POP_R3_TO_R1 break; \
  case 4:  ___POP_R4_TO_R1 break; \
  default: ___POP_R5_TO_R1 \
  }
#define ___SET_SELF(val)___SET_R6(val)
#define ___SELF ___R6
#define ___LD_ARG_REGS ___LD_R1_TO_R5
#define ___ST_ARG_REGS ___ST_R1_TO_R5
#endif

#if ___NB_GVM_REGS == 6

#define ___PUSH_REGS ___PUSH_R0 ___PUSH_R1_TO_R5
#define ___POP_REGS ___POP_R5_TO_R1 ___POP_R0

#else

#define ___PSR6 ___ps->r[6]
#define ___D_R6 register ___WORD ___r6;
#define ___R_R6 ___r6=___PSR6;
#define ___W_R6 ___PSR6=___r6;
#define ___R6 ___r6
#define ___SET_R6(val)___r6=(val);
#define ___PUSH_R1_TO_R6 ___PUSH_R1_TO_R5 ___PUSH(___R6)
#define ___POP_R6_TO_R1 ___SET_R6(___POP)___POP_R5_TO_R1
#define ___LD_R1_TO_R6 ___D_R6 ___LD_R1_TO_R5 ___R_R6
#define ___ST_R1_TO_R6 ___ST_R1_TO_R5 ___W_R6

#if ___NB_ARG_REGS == 6
#define ___PUSH_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___PUSH_R1_TO_R1 break; \
  case 2:  ___PUSH_R1_TO_R2 break; \
  case 3:  ___PUSH_R1_TO_R3 break; \
  case 4:  ___PUSH_R1_TO_R4 break; \
  case 5:  ___PUSH_R1_TO_R5 break; \
  default: ___PUSH_R1_TO_R6 \
  }
#define ___POP_ARGS_IN_REGS(na) \
switch (na) \
  { \
  case 0:  break; \
  case 1:  ___POP_R1_TO_R1 break; \
  case 2:  ___POP_R2_TO_R1 break; \
  case 3:  ___POP_R3_TO_R1 break; \
  case 4:  ___POP_R4_TO_R1 break; \
  case 5:  ___POP_R5_TO_R1 break; \
  default: ___POP_R6_TO_R1 \
  }
#define ___SET_SELF(val)___SET_R7(val)
#define ___SELF ___R7
#define ___LD_ARG_REGS ___LD_R1_TO_R6
#define ___ST_ARG_REGS ___ST_R1_TO_R6
#endif

#if ___NB_GVM_REGS == 7

#define ___PUSH_REGS ___PUSH_R0 ___PUSH_R1_TO_R6
#define ___POP_REGS ___POP_R6_TO_R1 ___POP_R0

#else

#define ___PSR7 ___ps->r[7]
#define ___D_R7 register ___WORD ___r7;
#define ___R_R7 ___r7=___PSR7;
#define ___W_R7 ___PSR7=___r7;
#define ___R7 ___r7
#define ___SET_R7(val)___r7=(val);
#define ___PUSH_R1_TO_R7 ___PUSH_R1_TO_R6 ___PUSH(___R7)
#define ___POP_R7_TO_R1 ___SET_R7(___POP)___POP_R6_TO_R1
#define ___LD_R1_TO_R7 ___D_R7 ___LD_R1_TO_R6 ___R_R7
#define ___ST_R1_TO_R7 ___ST_R1_TO_R6 ___W_R7

#if ___NB_GVM_REGS == 8

#define ___PUSH_REGS ___PUSH_R0 ___PUSH_R1_TO_R7
#define ___POP_REGS ___POP_R7_TO_R1 ___POP_R0

#endif
#endif
#endif
#endif
#endif
#endif
#endif

/*---------------------------------------------------------------------------*/

/* Labels and switches */

#define ___NOT(x)!x
#define ___AND(x,y)(x&&y)

#define ___LBL(n)___start+((___PH_LBL0-___HOST_LBL0+n)*___LS*___WS)
#define ___DEF_SLBL(n,lbl)case ___PH_LBL0-___HOST_LBL0+n:___DEF_GLBL(lbl)
#define ___DEF_GLBL(lbl)lbl:
#define ___GOTO(lbl)goto lbl;
#define ___IF_GOTO(test,lbl)if(test)___GOTO(lbl)
#define ___JUMPINT(nargs,prc,lbl)___SM(___GOTO(lbl),{nargs ___pc=prc;goto ___jump;})
#define ___JUMPEXTPRM(nargs,val){nargs ___pc=val;goto ___jumpext;}
#define ___JUMPEXT(nargs,val) \
{nargs ___SET_SELF(val);___pc=((___lbl_struct*)(___SELF-___tSUBTYPED))->entry;goto ___jumpext;}
#define ___JUMPPRM(nargs,val){nargs ___pc=val;goto ___jump;}

#define ___JUMP(nargs,val) \
{nargs ___SET_SELF(val); \
___pc=((___lbl_struct*)(___SELF-___tSUBTYPED))->entry;goto ___jump;}

#define ___JUMPGENNOTSAFE(nargs,val)___JUMP(nargs,val)
#define ___JUMPGLONOTSAFE(nargs,i,glo)___JUMP(nargs,___GLO(i,glo))
#define ___JUMPGENSAFE(nargs,val) \
{nargs ___SET_SELF(val); \
if (___PROCEDUREP(___SELF)) \
  {___pc=((___lbl_struct*)(___SELF-___tSUBTYPED))->entry;goto ___jump;} \
___ps->temp1=___SELF;___JUMPEXTPRM(___NOTHING,___ps->handler_not_proc)}
#define ___JUMPGLOSAFE(nargs,i,glo) \
{nargs ___SET_SELF(___GLO(i,glo)); \
if (___PROCEDUREP(___SELF)) \
  {___pc=((___lbl_struct*)(___SELF-___tSUBTYPED))->entry;goto ___jump;} \
___ps->temp4=&___GLOSTRUCT(i,glo);___JUMPEXTPRM(___NOTHING,___ps->handler_not_proc_glo)}

#ifdef ___NOT_SAFE_CALLS
#undef ___JUMPGENSAFE
#undef ___JUMPGLOSAFE
#define ___JUMPGENSAFE(nargs,val)___JUMPGENNOTSAFE(nargs,val)
#define ___JUMPGLOSAFE(nargs,i,glo)___JUMPGLONOTSAFE(nargs,i,glo)
#endif

#ifdef ___SAFE_CALLS
#undef ___JUMPGENNOTSAFE
#undef ___JUMPGLONOTSAFE
#define ___JUMPGENNOTSAFE(nargs,val)___JUMPGENSAFE(nargs,val)
#define ___JUMPGLONOTSAFE(nargs,i,glo)___JUMPGLOSAFE(nargs,i,glo)
#endif

#ifdef ___SINGLE_HOST

#define ___BEGIN_M_SW ___BEGIN_SW
#define ___END_M_SW ___END_SW
#define ___BEGIN_P_SW
#define ___END_P_SW

#else

#define ___BEGIN_M_SW
#define ___END_M_SW
#define ___BEGIN_P_SW ___BEGIN_SW
#define ___END_P_SW ___END_SW

#endif

#define ___BEGIN_SW \
___pc=___ps->pc; \
___jump: \
switch((___pc-=___start)/(___LS*___WS)) \
{
#define ___END_SW \
}___pc+=___start;___jumpext:

/*---------------------------------------------------------------------------*/

#define ___PRC(i)___start+((i-___HOST_LBL0)*___LS*___WS)

#define ___SET_NARGS(n)___ps->na=n;
#define ___IF_NARGS_EQ(n,code)if(___ps->na==n){code}else
#define ___WRONG_NARGS(lbl,nb_req,nb_opt,nb_key) \
{___ps->temp1=___LBL(lbl); \
___JUMPEXTPRM(___NOTHING,___ps->handler_wrong_nargs)}
#define ___GET_REST(lbl,nb_req,nb_opt,nb_key) \
if(___ps->na>=0){___ps->temp1=___LBL(lbl); \
___JUMPEXTPRM(___NOTHING,___ps->handler_get_rest)}
#define ___GET_KEY(lbl,nb_req,nb_opt,nb_key,key_descr) \
if(___ps->na>=0){___ps->temp1=___LBL(lbl);___ps->temp2=nb_req+nb_opt; \
___ps->temp3=key_descr;___JUMPEXTPRM(___NOTHING,___ps->handler_get_key)}
#define ___GET_KEY_REST(lbl,nb_req,nb_opt,nb_key,key_descr) \
if(___ps->na>=0){___ps->temp1=___LBL(lbl);___ps->temp2=nb_req+nb_opt; \
___ps->temp3=key_descr;___JUMPEXTPRM(___NOTHING,___ps->handler_get_key_rest)}

#define ___BOOL(x)(x)?___TRU:___FAL

#define ___ALLOC_CLO(n)(___hp+=n+2,___TAG((___hp-n-2),___tSUBTYPED))
#define ___SETUP_CLO(n,clo,lbl,elems){___WORD *___ptr=___UNTAG_AS(clo,___tSUBTYPED); *___ptr++=___MAKE_HD_WORDS((n+1),___sPROCEDURE); *___ptr++=___LBL(lbl); elems}
#define ___ADD_CLO_ELEM(val)*___ptr++=(val);
#define ___CLO(x,y)___BODY_AS(x,___tSUBTYPED)[y]
#define ___SET_CLO(x,y,z)___BODY_AS(x,___tSUBTYPED)[y]=z;

#define ___ALLOC_LIST(n,last,elems)___CONS(last,___NUL);elems
#define ___ADD_LIST_ELEM(val)___CONS(val,___TAG((___hp-3),___tPAIR));
#define ___GET_LIST(n) ___TAG((___hp-3),___tPAIR)

#define ___ALLOC_STRING(n,elems) \
___hp[0]=___MAKE_HD_BYTES((n<<___LCS),___sSTRING); \
{___C *___ptr=(___C*)(___hp+1); ___hp+=___WORDS((n<<___LCS))+1; \
elems}
#define ___ADD_STRING_ELEM(val)*___ptr++=___INT(val);
#define ___GET_STRING(n)___TAG((___hp-___WORDS((n<<___LCS))-1),___tSUBTYPED)

#define ___ALLOC_VECTOR8(n,elems) \
___hp[0]=___MAKE_HD_BYTES(n,___sVECTOR8); \
{___U8 *___ptr=(___U8*)(___hp+1); ___hp+=___WORDS(n)+1; \
elems}
#define ___ADD_VECTOR8_ELEM(val)*___ptr++=___INT(val);
#define ___GET_VECTOR8(n)___TAG((___hp-___WORDS(n)-1),___tSUBTYPED)

#define ___ALLOC_VECTOR16(n,elems) \
___hp[0]=___MAKE_HD_BYTES((n<<1),___sVECTOR16); \
{___U16 *___ptr=(___U16*)(___hp+1); ___hp+=___WORDS((n<<1))+1; \
elems}
#define ___ADD_VECTOR16_ELEM(val)*___ptr++=___INT(val);
#define ___GET_VECTOR16(n)___temp=___TAG((___hp-___WORDS((n<<1))-1),___tSUBTYPED)

#define ___ALLOC_VECTOR(n,elems) \
___hp[0]=___MAKE_HD_WORDS(n,___sVECTOR); \
{___WORD *___ptr=___hp+1; ___hp+=n+1; \
elems}
#define ___ADD_VECTOR_ELEM(val)*___ptr++=(val);
#define ___GET_VECTOR(n)___temp=___TAG((___hp-n-1),___tSUBTYPED)

#define ___POLL(n)if(___fp<___ps->stack_trip) \
{___ps->temp1=___LBL(n);___JUMPEXTPRM(___NOTHING,___ps->handler_stack_limit)}

#define ___TASK_PUSH(i)
#define ___TASK_POP(n)

#define ___CHECK_HEAP(n,m)if(___hp>___ps->heap_limit) \
{___ps->temp1=___LBL(n);___JUMPEXTPRM(___NOTHING,___ps->handler_heap_limit)}

#define ___FORCE1(n,src) \
if (___TYP((___temp=(src)))==___tSUBTYPED&&___SUBTYPE(___temp)==___sPLACEHOLDER<<___TB){___ps->temp1=___LBL(n);___ps->temp2=___temp;___JUMPEXTPRM(___NOTHING,___ps->handler_force);
#define ___FORCE2 ___temp=___ps->temp2;}
#define ___FORCE3 ___temp

#ifndef ___NO_DYNAMIC_IMPORTS

#define ___GSTATE_IMPORTS

#else

#ifdef ___BIND_LATE
#define ___IMPORTS_VIA_GSTATE
#define ___LOCAL_GSTATE
#endif

#define ___GSTATE_IMPORTS \
double (*fabs) (); \
double (*floor) (); \
double (*ceil) (); \
double (*exp) (); \
double (*log) (); \
double (*sin) (); \
double (*cos) (); \
double (*tan) (); \
double (*asin) (); \
double (*acos) (); \
double (*atan) (); \
double (*sqrt) (); \
int (*___scmobj_to_char) (); \
int (*___scmobj_to_schar) (); \
int (*___scmobj_to_uchar) (); \
int (*___scmobj_to_latin1) (); \
int (*___scmobj_to_ucs4) (); \
int (*___scmobj_to_ucs2) (); \
int (*___scmobj_to_short) (); \
int (*___scmobj_to_ushort) (); \
int (*___scmobj_to_int) (); \
int (*___scmobj_to_uint) (); \
int (*___scmobj_to_long) (); \
int (*___scmobj_to_ulong) (); \
int (*___scmobj_to_float) (); \
int (*___scmobj_to_double) (); \
int (*___scmobj_to_pointer) (); \
int (*___scmobj_to_function) (); \
int (*___scmobj_to_boolean) (); \
int (*___scmobj_to_charstring) (); \
int (*___scmobj_to_latin1string) (); \
int (*___scmobj_to_ucs4string) (); \
int (*___scmobj_to_ucs2string) (); \
int (*___scmobj_to_utf8string) (); \
void (*___free_function) (); \
void (*___free_charstring) (); \
void (*___free_latin1string) (); \
void (*___free_ucs4string) (); \
void (*___free_ucs2string) (); \
void (*___free_utf8string) (); \
int (*___char_to_scmobj) (); \
int (*___schar_to_scmobj) (); \
int (*___uchar_to_scmobj) (); \
int (*___latin1_to_scmobj) (); \
int (*___ucs4_to_scmobj) (); \
int (*___ucs2_to_scmobj) (); \
int (*___short_to_scmobj) (); \
int (*___ushort_to_scmobj) (); \
int (*___int_to_scmobj) (); \
int (*___uint_to_scmobj) (); \
int (*___long_to_scmobj) (); \
int (*___ulong_to_scmobj) (); \
int (*___float_to_scmobj) (); \
int (*___double_to_scmobj) (); \
int (*___pointer_to_scmobj) (); \
int (*___function_to_scmobj) (); \
int (*___boolean_to_scmobj) (); \
int (*___charstring_to_scmobj) (); \
int (*___latin1string_to_scmobj) (); \
int (*___ucs4string_to_scmobj) (); \
int (*___ucs2string_to_scmobj) (); \
int (*___utf8string_to_scmobj) (); \
void (*___release_scmobj) (); \
double (*___round) (); \
___WORD (*___alloc_scmobj) (); \
___WORD (*___make_pair) (); \
___WORD (*___make_vector) (); \
___WORD (*___make_string) (); \
void (*___still_obj_refcount_inc) (); \
void (*___still_obj_refcount_dec) ();

#endif

#ifdef ___IMPORTS_VIA_GSTATE

#define ___EXT(f)___GSTATE->f
#define ___MODULE_IMPORTS

#else

#define ___EXT(f)f
#define ___MODULE_IMPORTS \
extern double fabs (); \
extern double floor (); \
extern double ceil (); \
extern double exp (); \
extern double log (); \
extern double sin (); \
extern double cos (); \
extern double tan (); \
extern double asin (); \
extern double acos (); \
extern double atan (); \
extern double sqrt (); \
___EXPORT extern int ___scmobj_to_char (); \
___EXPORT extern int ___scmobj_to_schar (); \
___EXPORT extern int ___scmobj_to_uchar (); \
___EXPORT extern int ___scmobj_to_latin1 (); \
___EXPORT extern int ___scmobj_to_ucs4 (); \
___EXPORT extern int ___scmobj_to_ucs2 (); \
___EXPORT extern int ___scmobj_to_short (); \
___EXPORT extern int ___scmobj_to_ushort (); \
___EXPORT extern int ___scmobj_to_int (); \
___EXPORT extern int ___scmobj_to_uint (); \
___EXPORT extern int ___scmobj_to_long (); \
___EXPORT extern int ___scmobj_to_ulong (); \
___EXPORT extern int ___scmobj_to_float (); \
___EXPORT extern int ___scmobj_to_double (); \
___EXPORT extern int ___scmobj_to_pointer (); \
___EXPORT extern int ___scmobj_to_function (); \
___EXPORT extern int ___scmobj_to_boolean (); \
___EXPORT extern int ___scmobj_to_charstring (); \
___EXPORT extern int ___scmobj_to_latin1string (); \
___EXPORT extern int ___scmobj_to_ucs4string (); \
___EXPORT extern int ___scmobj_to_ucs2string (); \
___EXPORT extern int ___scmobj_to_utf8string (); \
___EXPORT extern void ___free_function (); \
___EXPORT extern void ___free_charstring (); \
___EXPORT extern void ___free_latin1string (); \
___EXPORT extern void ___free_ucs4string (); \
___EXPORT extern void ___free_ucs2string (); \
___EXPORT extern void ___free_utf8string (); \
___EXPORT extern int ___char_to_scmobj (); \
___EXPORT extern int ___schar_to_scmobj (); \
___EXPORT extern int ___uchar_to_scmobj (); \
___EXPORT extern int ___latin1_to_scmobj (); \
___EXPORT extern int ___ucs4_to_scmobj (); \
___EXPORT extern int ___ucs2_to_scmobj (); \
___EXPORT extern int ___short_to_scmobj (); \
___EXPORT extern int ___ushort_to_scmobj (); \
___EXPORT extern int ___int_to_scmobj (); \
___EXPORT extern int ___uint_to_scmobj (); \
___EXPORT extern int ___long_to_scmobj (); \
___EXPORT extern int ___ulong_to_scmobj (); \
___EXPORT extern int ___float_to_scmobj (); \
___EXPORT extern int ___double_to_scmobj (); \
___EXPORT extern int ___pointer_to_scmobj (); \
___EXPORT extern int ___function_to_scmobj (); \
___EXPORT extern int ___boolean_to_scmobj (); \
___EXPORT extern int ___charstring_to_scmobj (); \
___EXPORT extern int ___latin1string_to_scmobj (); \
___EXPORT extern int ___ucs4string_to_scmobj (); \
___EXPORT extern int ___ucs2string_to_scmobj (); \
___EXPORT extern int ___utf8string_to_scmobj (); \
___EXPORT extern void ___release_scmobj (); \
___EXPORT extern double ___round (); \
___EXPORT extern ___WORD ___alloc_scmobj (); \
___EXPORT extern ___WORD ___make_pair (); \
___EXPORT extern ___WORD ___make_vector (); \
___EXPORT extern ___WORD ___make_string (); \
___EXPORT extern void ___still_obj_refcount_inc (); \
___EXPORT extern void ___still_obj_refcount_dec ();

#endif

#define ___NB_INTRS 3
#define ___INTR_USER  0
#define ___INTR_TIMER 1
#define ___INTR_GC    2

#define ___MODULE_TYPEDEFS \
typedef struct ___glo_struct { ___WORD val, prm; struct ___glo_struct *next; } ___glo_struct; \
typedef struct \
{ \
___WORD *stack, *stack_base, *stack_limit, *stack_trip, *stack_break, *fp; \
___WORD *heap, *heap_limit, *hp; \
___WORD r[___NB_GVM_REGS], pc, temp1, temp2, temp3; ___glo_struct *temp4; \
int na, np; \
int intr_enabled, intr_flag[___NB_INTRS]; \
___glo_struct *glo_list; \
___WORD \
handler_break, handler_stack_limit, handler_heap_limit, \
handler_not_proc, handler_not_proc_glo, \
handler_wrong_nargs, handler_get_rest, handler_get_key, handler_get_key_rest, \
handler_force, handler_c_conv_error; \
___WORD executable_wills, non_executable_wills; \
} ___processor_struct; \
typedef ___processor_struct *___processor; \
typedef struct \
{ \
___processor_struct processor_state; \
___GSTATE_IMPORTS \
} ___global_struct; \
typedef ___WORD (*___host) (); \
typedef struct { ___WORD header, entry, flags; ___FAKEHOST host; } ___lbl_struct; \
struct ___module \
{ \
int version; \
int kind; \
___UTF8STRING name; \
___glo_struct **glo_tbl; int glo_count; int sup_count; \
___UTF8STRING *glo_names; \
___FAKEWORD *sym_tbl; int sym_count; \
___UTF8STRING *sym_names; \
___FAKEWORD *key_tbl; int key_count; \
___UTF8STRING *key_names; \
___WORD *lp; \
___lbl_struct *lbl_tbl; int lbl_count; \
___WORD *cns_tbl; int cns_count; \
___FAKEWORD *sub_tbl; int sub_count; \
void (*init_proc) (); \
struct ___module *next; \
}; \
struct ___linkfile \
{ \
int version; \
int kind; \
___UTF8STRING name; \
___FAKEWORD *sym_list; \
___FAKEWORD *key_list; \
void **linker_tbl; \
}; \
typedef union \
{ \
struct ___module module; \
struct ___linkfile linkfile; \
} *___mod_or_lnk; \
typedef struct \
  { \
    int argc; \
    char **argv; \
    unsigned long min_heap; \
    unsigned long max_heap; \
    long (*gc_hook) (); \
    void (*fatal_error) (); \
    int debug_level; \
    int default_io_encoding; \
    int force_tty; \
    ___mod_or_lnk (*linker) (); \
  } ___setup_params;

#define ___IO_DEFAULT_ENCODING 0
#define ___IO_CHAR_ENCODING    1
#define ___IO_LATIN1_ENCODING  2
#define ___IO_UCS4_ENCODING    3
#define ___IO_UCS2_ENCODING    4
#define ___IO_UTF8_ENCODING    5

#define ___MODULE_KIND 0
#define ___LINKFILE_KIND 1

#define ___SPECIAL_DECL

#ifdef _MSC_VER
#ifdef ___SHARED
#ifdef ___BIND_LATE
#undef ___SPECIAL_DECL
#define ___SPECIAL_DECL int _fltused; /* needed if floating-pt used. */
#endif
#endif
#endif

#define ___BEGIN_MOD_OR_LNK \
___MODULE_TYPEDEFS \
___MODULE_IMPORTS \
___GSTATE_DECL \
___SPECIAL_DECL \
___EXPORT extern void ___main (); \
___EXPORT extern void ___setup_and_run (); \
___EXPORT extern void ___cleanup (); \
___EXPORT extern int ___call (); \
___EXPORT extern void ___propagate_error (); \
___LOCAL ___WORD ___lp; \
___GLO_DECL \
___SYM_DECL \
___KEY_DECL

#ifdef ___LOCAL_GSTATE

#define ___GSTATE ___local_gstate
#define ___GSTATE_DECL ___LOCAL ___global_struct *___local_gstate;
#define ___SET_LOCAL_GSTATE(gs) ___local_gstate = gs;

#else

#define ___GSTATE (&___gstate)
#define ___GSTATE_DECL ___IMPORT extern ___global_struct ___gstate;
#define ___SET_LOCAL_GSTATE(gs)

#endif

#define ___PSTATE (&___GSTATE->processor_state)

#define ___BEGIN_MODULE ___BEGIN_MOD_OR_LNK
#define ___BEGIN_LINKFILE ___BEGIN_MOD_OR_LNK

#ifdef ___BIND_LATE

#define ___NEED_SYM(sym)
#define ___NEED_KEY(key)
#define ___NEED_GLO(glo)

#define ___BEGIN_SYM1 ___LOCAL ___UTF8STRING ___sym_names[]={
#define ___DEF_SYM1(i,sym,str)str,
#define ___END_SYM1 0};

#define ___DEF_SYM2(i,sym,str)

#define ___BEGIN_KEY1 ___LOCAL ___UTF8STRING ___key_names[]={
#define ___DEF_KEY1(i,key,str)str,
#define ___END_KEY1 0};

#define ___DEF_KEY2(i,key,str)

#define ___BEGIN_GLO ___LOCAL ___UTF8STRING ___glo_names[]={
#define ___DEF_GLO(i,str)str,
#define ___END_GLO 0};

#define ___BEGIN_OLD_KEY
#define ___DEF_OLD_KEY(key)
#define ___END_OLD_KEY

#define ___BEGIN_OLD_SYM_GLO
#define ___DEF_OLD_SYM_GLO(sym,glo)
#define ___END_OLD_SYM_GLO

#define ___BEGIN_NEW_KEY
#define ___DEF_NEW_KEY(prevkey,key,str,hash)
#define ___END_NEW_KEY

#define ___BEGIN_NEW_SYM_GLO
#define ___DEF_NEW_SYM_GLO_SUP(prevsym,sym,str,hash,glo)
#define ___DEF_NEW_SYM_GLO(prevsym,sym,str,hash,glo)
#define ___END_NEW_SYM_GLO(prevsym,prevkey)___LINKFILE_DESCR(0,0)

#define ___DEF_SUB_NSTR(id,n)
#define ___NSTR0
#define ___NSTR1(a)
#define ___NSTR2(a,b)
#define ___NSTR3(a,b,c)
#define ___NSTR4(a,b,c,d)
#define ___NSTR5(a,b,c,d,e)
#define ___NSTR6(a,b,c,d,e,f)
#define ___NSTR7(a,b,c,d,e,f,g)
#define ___NSTR8(a,b,c,d,e,f,g,h)

#else

#define ___NEED_SYM(sym)___IMPORT extern ___WORD sym[];
#define ___NEED_KEY(key)___IMPORT extern ___WORD key[];
#define ___NEED_GLO(glo)___IMPORT extern ___glo_struct glo;

#ifndef _MSC_VER

#define ___BEGIN_SYM1 ___LOCAL ___FAKEWORD ___sym_tbl[]={
#define ___DEF_SYM1(i,sym,str)(___FAKEWORD)(sym),
#define ___END_SYM1 0};

#define ___DEF_SYM2(i,sym,str)

#define ___BEGIN_KEY1 ___LOCAL ___FAKEWORD ___key_tbl[]={
#define ___DEF_KEY1(i,key,str)(___FAKEWORD)(key),
#define ___END_KEY1 0};

#define ___DEF_KEY2(i,key,str)

#else

#define ___BEGIN_SYM1 ___LOCAL ___FAKEWORD ___sym_tbl[___SYM_COUNT];
#define ___DEF_SYM1(i,sym,str)
#define ___END_SYM1

#define ___DEF_SYM2(i,sym,str)___sym_tbl[i]=(___FAKEWORD)(sym);

#define ___BEGIN_KEY1 ___LOCAL ___FAKEWORD ___key_tbl[___KEY_COUNT];
#define ___DEF_KEY1(i,key,str)
#define ___END_KEY1

#define ___DEF_KEY2(i,key,str)___key_tbl[i]=(___FAKEWORD)(key);

#endif

#define ___BEGIN_GLO
#define ___DEF_GLO(i,str)
#define ___END_GLO

#define ___BEGIN_OLD_KEY
#define ___DEF_OLD_KEY(key)
#define ___END_OLD_KEY

#define ___BEGIN_OLD_SYM_GLO
#define ___DEF_OLD_SYM_GLO(sym,glo)
#define ___END_OLD_SYM_GLO

#define ___BEGIN_NEW_KEY
#define ___DEF_NEW_KEY(prevkey,key,str,hash) \
___EXPORT ___FAKEWORD key[]={(___FAKEWORD)prevsym,(___FAKEWORD)str ___PADDING};
#define ___END_NEW_KEY

#define ___BEGIN_NEW_SYM_GLO
#define ___DEF_NEW_SYM_GLO_SUP(prevsym,sym,str,hash,glo) \
___EXPORT ___glo_struct glo = {___UNB2, ___FIX(hash), 0}; \
___EXPORT ___FAKEWORD sym[]={(___FAKEWORD)prevsym,(___FAKEWORD)str,0,(___FAKEWORD)&glo ___PADDING};
#define ___DEF_NEW_SYM_GLO(prevsym,sym,str,hash,glo) \
___EXPORT ___glo_struct glo = {___UNB1, ___FIX(hash), 0}; \
___EXPORT ___FAKEWORD sym[]={(___FAKEWORD)prevsym,(___FAKEWORD)str,0,(___FAKEWORD)&glo ___PADDING};
#define ___END_NEW_SYM_GLO(prevsym,prevkey)___LINKFILE_DESCR(prevsym,prevkey)

#define ___DEF_SUB_NSTR(id,n) ___DEF_SUB_STR(id,n)
#define ___NSTR0 ___STR0
#define ___NSTR1(a) ___STR1(a)
#define ___NSTR2(a,b) ___STR2(a,b)
#define ___NSTR3(a,b,c) ___STR3(a,b,c)
#define ___NSTR4(a,b,c,d) ___STR4(a,b,c,d)
#define ___NSTR5(a,b,c,d,e) ___STR5(a,b,c,d,e)
#define ___NSTR6(a,b,c,d,e,f) ___STR6(a,b,c,d,e,f)
#define ___NSTR7(a,b,c,d,e,f,g) ___STR7(a,b,c,d,e,f,g)
#define ___NSTR8(a,b,c,d,e,f,g,h) ___STR8(a,b,c,d,e,f,g,h)

#endif

#define ___BEGIN_CNS \
___LOCAL ___WORD ___cns_tbl[]={
#define ___DEF_CNS(car,cdr)___MAKE_HD((___PAIR_SIZE<<___LWS),___sPAIR,___RCN,___PERM),cdr,car
#define ___END_CNS ___PADDING};

#define ___DEF_SUB_VEC(id,n) \
___LOCAL ___WORD id[]={___MAKE_HD((n<<___LWS),___sVECTOR,___RCN,___PERM)

#define ___DEF_SUB_STR(id,n) \
___LOCAL ___WORD id[]={___MAKE_HD((n<<___LCS),___sSTRING,___RCN,___PERM)

#define ___DEF_SUB_BIG(id,n) \
___LOCAL ___WORD id[]={___MAKE_HD((n<<1),___sBIGNUM,___RCN,___PERM)

#define ___DEF_SUB_RAT(id,num,den) \
___LOCAL ___WORD id[]={___MAKE_HD((___RATNUM_SIZE<<___LWS),___sRATNUM,___RCN,___PERM),num,den ___PADDING};

#define ___DEF_SUB_FLO(id,hi,lo) \
___LOCAL ___WORD id[]={___MAKE_HD((___FLONUM_SIZE<<___LWS),___sFLONUM,___RCN,___PERM), \
___FLO2(hi,lo) ___PADDING_FLO};

#if ___FLONUM_SIZE == 1

#define ___FLO2(hi,lo)((___WORD)hi<<32)+(___U32)lo

#else

#ifdef ___CPU_BIGEND
#define ___FLO2(hi,lo)hi,lo
#else
#define ___FLO2(hi,lo)lo,hi
#endif

#endif

#define ___DEF_SUB_CPX(id,real,imag) \
___LOCAL ___WORD id[]={___MAKE_HD((___CPXNUM_SIZE<<___LWS),___sCPXNUM,___RCN,___PERM),real,imag ___PADDING};

#define ___BEGIN_SUB \
___LOCAL ___FAKEWORD ___sub_tbl[]={
#define ___DEF_SUB(id)___CAST_WORDSTAR_TO_FAKEWORD(id)
#define ___END_SUB };

#define ___VEC0 ___PADDING};
#define ___VEC1(a),a

#if ___WS == 4

#if ___CS == 1

#ifdef ___CPU_BIGEND
#define ___S4(a,b,c,d)((((___WORD)(___C)a<<8)+(___WORD)(___C)b<<8)+(___WORD)(___C)c<<8)+(___WORD)(___C)d
#else
#define ___S4(d,c,b,a)((((___WORD)(___C)a<<8)+(___WORD)(___C)b<<8)+(___WORD)(___C)c<<8)+(___WORD)(___C)d
#endif

#else

#if ___CS == 2

#ifdef ___CPU_BIGEND
#define ___S4(a,b,c,d)((___WORD)(___C)a<<16)+(___WORD)(___C)b,((___WORD)(___C)c<<16)+(___WORD)(___C)d
#else
#define ___S4(b,a,d,c)((___WORD)(___C)a<<16)+(___WORD)(___C)b,((___WORD)(___C)c<<16)+(___WORD)(___C)d
#endif

#else

#define ___S4(a,b,c,d)a,b,c,d

#endif

#endif

#define ___STR0 ___PADDING};
#define ___STR1(a),___S4(a,0,0,0) ___PADDING};
#define ___STR2(a,b),___S4(a,b,0,0) ___PADDING};
#define ___STR3(a,b,c),___S4(a,b,c,0) ___PADDING};
#define ___STR4(a,b,c,d),___S4(a,b,c,d) ___PADDING};
#define ___STR5(a,b,c,d,e),___S4(a,b,c,d),___S4(e,0,0,0) ___PADDING};
#define ___STR6(a,b,c,d,e,f),___S4(a,b,c,d),___S4(e,f,0,0) ___PADDING};
#define ___STR7(a,b,c,d,e,f,g),___S4(a,b,c,d),___S4(e,f,g,0) ___PADDING};
#define ___STR8(a,b,c,d,e,f,g,h),___S4(a,b,c,d),___S4(e,f,g,h)

#else

#if ___CS == 1

#ifdef ___CPU_BIGEND
#define ___S8(a,b,c,d,e,f,g,h)(((((((((___WORD)(___C)a<<8)+(___WORD)(___C)b<<8)+(___WORD)(___C)c<<8)+(___WORD)(___C)d<<8)+(___WORD)(___C)e<<8)+(___WORD)(___C)f<<8)+(___WORD)(___C)g<<8)+(___WORD)(___C)h)
#else
#define ___S8(h,g,f,e,d,c,b,a)(((((((((___WORD)(___C)a<<8)+(___WORD)(___C)b<<8)+(___WORD)(___C)c<<8)+(___WORD)(___C)d<<8)+(___WORD)(___C)e<<8)+(___WORD)(___C)f<<8)+(___WORD)(___C)g<<8)+(___WORD)(___C)h)
#endif

#else

#if ___CS == 2

#ifdef ___CPU_BIGEND
#define ___S8(a,b,c,d,e,f,g,h)((((___WORD)(___C)a<<16)+(___WORD)(___C)b<<16)+(___WORD)(___C)c<<16)+(___WORD)(___C)d,((((___WORD)(___C)e<<16)+(___WORD)(___C)f<<16)+(___WORD)(___C)g<<16)+(___WORD)(___C)h
#else
#define ___S8(d,c,b,a,h,g,f,e)((((___WORD)(___C)a<<16)+(___WORD)(___C)b<<16)+(___WORD)(___C)c<<16)+(___WORD)(___C)d,((((___WORD)(___C)e<<16)+(___WORD)(___C)f<<16)+(___WORD)(___C)g<<16)+(___WORD)(___C)h
#endif

#else

#ifdef ___CPU_BIGEND
#define ___S8(a,b,c,d,e,f,g,h)((___WORD)(___C)a<<32)+(___WORD)(___C)b,((___WORD)(___C)c<<32)+(___WORD)(___C)d,((___WORD)(___C)e<<32)+(___WORD)(___C)f,((___WORD)(___C)g<<32)+(___WORD)(___C)h
#else
#define ___S8(b,a,d,c,f,e,h,g)((___WORD)(___C)a<<32)+(___WORD)(___C)b,((___WORD)(___C)c<<32)+(___WORD)(___C)d,((___WORD)(___C)e<<32)+(___WORD)(___C)f,((___WORD)(___C)g<<32)+(___WORD)(___C)h
#endif

#endif

#endif

#define ___STR0 ___PADDING};
#define ___STR1(a),___S8(a,0,0,0,0,0,0,0) ___PADDING};
#define ___STR2(a,b),___S8(a,b,0,0,0,0,0,0) ___PADDING};
#define ___STR3(a,b,c),___S8(a,b,c,0,0,0,0,0) ___PADDING};
#define ___STR4(a,b,c,d),___S8(a,b,c,d,0,0,0,0) ___PADDING};
#define ___STR5(a,b,c,d,e),___S8(a,b,c,d,e,0,0,0) ___PADDING};
#define ___STR6(a,b,c,d,e,f),___S8(a,b,c,d,e,f,0,0) ___PADDING};
#define ___STR7(a,b,c,d,e,f,g),___S8(a,b,c,d,e,f,g,0) ___PADDING};
#define ___STR8(a,b,c,d,e,f,g,h),___S8(a,b,c,d,e,f,g,h)

#endif

#if ___WS == 4

#ifdef ___CPU_BIGEND
#define ___B2(a,b)((___WORD)(___U16)a<<16)+(___WORD)(___U16)b
#else
#define ___B2(b,a)((___WORD)(___U16)a<<16)+(___WORD)(___U16)b
#endif

#define ___BIG0 ___PADDING};
#define ___BIG1(a),___B2(a,0) ___PADDING};
#define ___BIG2(a,b),___B2(a,b) ___PADDING};
#define ___BIG3(a,b,c),___B2(a,b),___B2(c,0) ___PADDING};
#define ___BIG4(a,b,c,d),___B2(a,b),___B2(c,d)

#else

#ifdef ___CPU_BIGEND
#define ___B4(a,b,c,d)((((___WORD)(___U16)a<<16)+(___WORD)(___U16)b<<16)+(___WORD)(___U16)c<<16)+(___WORD)(___U16)d
#else
#define ___B4(d,c,b,a)((((___WORD)(___U16)a<<16)+(___WORD)(___U16)b<<16)+(___WORD)(___U16)c<<16)+(___WORD)(___U16)d
#endif

#define ___BIG0 ___PADDING};
#define ___BIG1(a),___B4(a,0,0,0) ___PADDING};
#define ___BIG2(a,b),___B4(a,b,0,0) ___PADDING};
#define ___BIG3(a,b,c),___B4(a,b,c,0) ___PADDING};
#define ___BIG4(a,b,c,d),___B4(a,b,c,d)

#endif

#define ___MAKE_PROC_INFO(nbparms,nbclosed)(nbclosed<<17)+(nbparms<<3)+1
#define ___PROC_INFO_NBPARMS(x)(((x)>>3)&((1<<14)-1))
#define ___PROC_INFO_NBCLOSED(x)(((x)>>17)&((1<<14)-1))

#define ___MAKE_RETN_INFO(fs,link)(link<<17)+(fs<<3)+3
#define ___RETN_INFO_FS(x)(((x)>>3)&((1<<14)-1))
#define ___RETN_INFO_LINK(x)(((x)>>17)&((1<<14)-1))

#define ___MAKE_RETT_INFO(fs,link)(link<<17)+(fs<<3)+4
#define ___RETT_INFO_FS(x)(((x)>>3)&((1<<14)-1))
#define ___RETT_INFO_LINK(x)(((x)>>17)&((1<<14)-1))

#define ___MAKE_RETI_INFO(fs,link)(link<<17)+((fs+___NB_GVM_REGS+1)<<3)+7
#define ___RETI_INFO_FS(x)(((x)>>3)&((1<<14)-1))
#define ___RETI_INFO_LINK(x)(((x)>>17)&((1<<14)-1))

#define ___BEGIN_LBL \
___LOCAL ___lbl_struct ___lbl_tbl[]={
#define ___DEF_LBL_INTRO(id,info,cproc){___MAKE_HD((___INTRO_SIZE<<___LWS),___sVECTOR,___RCN,___PERM),info,0,(___host)cproc}
#define ___DEF_LBL_PROC(id,nbp,nbc){___MAKE_HD((3<<___LWS),___sPROCEDURE,___RCN,___PERM),0,___MAKE_PROC_INFO(nbp,nbc),___CAST_HOST_TO_FAKEHOST(___SM(___MH_PROC,id))}
#define ___DEF_LBL_RETN(id,fs,link){___MAKE_HD((3<<___LWS),___sPROCEDURE,___RCN,___PERM),0,___MAKE_RETN_INFO(fs,link),___CAST_HOST_TO_FAKEHOST(___SM(___MH_PROC,id))}
#define ___DEF_LBL_RETT(id,fs,link){___MAKE_HD((3<<___LWS),___sPROCEDURE,___RCN,___PERM),0,___MAKE_RETT_INFO(fs,link),___CAST_HOST_TO_FAKEHOST(___SM(___MH_PROC,id))}
#define ___DEF_LBL_RETI(id,fs,link){___MAKE_HD((3<<___LWS),___sPROCEDURE,___RCN,___PERM),0,___MAKE_RETI_INFO(fs,link),___CAST_HOST_TO_FAKEHOST(___SM(___MH_PROC,id))}
#define ___END_LBL ___PADDING_LBL};

#ifdef ___GLO_COUNT
#ifdef ___BIND_LATE
#define ___GLO_DECL ___LOCAL ___glo_struct *___glo_tbl[___GLO_COUNT];
#define ___GLO_TBL ___glo_tbl,___GLO_COUNT,___SUP_COUNT,___glo_names
#else
#define ___GLO_DECL
#define ___GLO_TBL 0,0,0,0
#endif
#else
#define ___GLO_DECL
#define ___GLO_TBL 0,0,0,0
#endif

#ifdef ___SYM_COUNT
#ifdef ___BIND_LATE
#define ___SYM_DECL ___LOCAL ___FAKEWORD ___sym_tbl[___SYM_COUNT];
#define ___SYM_TBL ___sym_tbl,___SYM_COUNT,___sym_names
#else
#define ___SYM_DECL
#define ___SYM_TBL ___sym_tbl,___SYM_COUNT,0
#endif
#else
#define ___SYM_DECL
#define ___SYM_TBL 0,0,0
#endif

#ifdef ___KEY_COUNT
#ifdef ___BIND_LATE
#define ___KEY_DECL ___LOCAL ___FAKEWORD ___key_tbl[___KEY_COUNT];
#define ___KEY_TBL ___key_tbl,___KEY_COUNT,___key_names
#else
#define ___KEY_DECL
#define ___KEY_TBL ___key_tbl,___KEY_COUNT,0
#endif
#else
#define ___KEY_DECL
#define ___KEY_TBL 0,0,0
#endif

#ifdef ___LBL_COUNT
#define ___LBL_TBL &___lp,___lbl_tbl,___LBL_COUNT
#else
#define ___LBL_TBL 0,0,0
#endif

#ifdef ___CNS_COUNT
#define ___CNS_TBL ___cns_tbl,___CNS_COUNT
#else
#define ___CNS_TBL 0,0
#endif

#ifdef ___SUB_COUNT
#define ___SUB_TBL ___sub_tbl,___SUB_COUNT
#else
#define ___SUB_TBL 0,0
#endif

#ifndef ___HIDDEN
#define ___HIDDEN static
#endif

#ifndef ___LOCAL
#define ___LOCAL static
#endif

#ifndef ___SHARED
#undef ___EXPORT
#undef ___IMPORT
#endif

#ifndef ___EXPORT
#define ___EXPORT
#endif

#ifndef ___IMPORT
#define ___IMPORT
#endif

#define ___BEGIN_MOD1 ___LOCAL void ___init_proc(){
#define ___DEF_PRM(i,glo,n)___PRM(i,glo)=___GLO(i,glo)=___lp+(n*___LS*___WS);
#define ___END_MOD1 }

#define ___BEGIN_MOD2 ___LOCAL struct ___module ___module_descr={ \
___VERSION,___MODULE_KIND,___MODULE_NAME, \
___GLO_TBL,___SYM_TBL,___KEY_TBL,___LBL_TBL,___CNS_TBL,___SUB_TBL, \
___init_proc,0}; \
___EXPORT ___mod_or_lnk ___LINKER_ID (___gs) \
___global_struct *___gs; \
{ ___SET_LOCAL_GSTATE(___gs)
#define ___END_MOD2 return (___mod_or_lnk)&___module_descr;}

#ifndef ___LIBRARY
#define ___MAIN_PROC \
void main (argc, argv) \
int argc; char **argv; \
{ ___main (argc, argv, ___LINKER_ID); }
#else
#define ___MAIN_PROC
#endif

#define ___LINKFILE_DESCR(sym_list,key_list) \
___LOCAL struct ___linkfile ___linkfile_descr={ \
___VERSION,___LINKFILE_KIND,___LINKFILE_NAME, \
sym_list,key_list,___linker_tbl}; \
___EXPORT ___mod_or_lnk ___LINKER_ID (___gs) \
___global_struct *___gs; \
{ ___SET_LOCAL_GSTATE(___gs) return (___mod_or_lnk)&___linkfile_descr;} \
___MAIN_PROC

#define ___BEGIN_LNK1
#define ___DEF_LNK1(id)extern ___mod_or_lnk id ();
#define ___END_LNK1

#define ___BEGIN_LNK2 ___LOCAL void *___linker_tbl[]={
#define ___DEF_LNK2(id)(void*)id
#define ___END_LNK2 ,0};


#define ___NO_ERR                 0
#define ___STOC_CHAR_ERR          1
#define ___STOC_SCHAR_ERR         2
#define ___STOC_UCHAR_ERR         3
#define ___STOC_LATIN1_ERR        4
#define ___STOC_UCS4_ERR          5
#define ___STOC_UCS2_ERR          6
#define ___STOC_SHORT_ERR         7
#define ___STOC_USHORT_ERR        8
#define ___STOC_INT_ERR           9
#define ___STOC_UINT_ERR          10
#define ___STOC_LONG_ERR          11
#define ___STOC_ULONG_ERR         12
#define ___STOC_FLOAT_ERR         13
#define ___STOC_DOUBLE_ERR        14
#define ___STOC_POINTER_ERR       15
#define ___STOC_FUNCTION_ERR      16
#define ___STOC_BOOLEAN_ERR       17
#define ___STOC_CHARSTRING_ERR    18
#define ___STOC_LATIN1STRING_ERR  19
#define ___STOC_UCS4STRING_ERR    20
#define ___STOC_UCS2STRING_ERR    21
#define ___STOC_UTF8STRING_ERR    22
#define ___STOC_HEAP_OVERFLOW_ERR 23
#define ___CTOS_CHAR_ERR          31
#define ___CTOS_SCHAR_ERR         32
#define ___CTOS_UCHAR_ERR         33
#define ___CTOS_LATIN1_ERR        34
#define ___CTOS_UCS4_ERR          35
#define ___CTOS_UCS2_ERR          36
#define ___CTOS_SHORT_ERR         37
#define ___CTOS_USHORT_ERR        38
#define ___CTOS_INT_ERR           39
#define ___CTOS_UINT_ERR          40
#define ___CTOS_LONG_ERR          41
#define ___CTOS_ULONG_ERR         42
#define ___CTOS_FLOAT_ERR         43
#define ___CTOS_DOUBLE_ERR        44
#define ___CTOS_POINTER_ERR       45
#define ___CTOS_FUNCTION_ERR      46
#define ___CTOS_BOOLEAN_ERR       47
#define ___CTOS_CHARSTRING_ERR    48
#define ___CTOS_LATIN1STRING_ERR  49
#define ___CTOS_UCS4STRING_ERR    50
#define ___CTOS_UCS2STRING_ERR    51
#define ___CTOS_UTF8STRING_ERR    52
#define ___CTOS_HEAP_OVERFLOW_ERR 53

#define ___C_CONV_ERROR {___ps->temp1=___LBL(0);___JUMPEXTPRM(___SET_NARGS(___NARGS),___ps->handler_c_conv_error)}

#define ___STOC(f,s,c)if ((___err=___EXT(f)(s,&c))==___NO_ERR)
#define ___CTOS(f,c,s)if (!(___err=___EXT(f)(c,&s)))


#define ___BEGIN_CDEF_VOID {___processor ___ps=___PSTATE;int ___err=___NO_ERR;
#define ___END_CDEF_VOID if (___err) ___propagate_error (___err); }
#define ___CDEF_CALL_VOID(lbl)___call (___NARGS,___lp+(lbl)*___LS*___WS);
#define ___CDEF_EXIT_VOID

#define ___BEGIN_CDEF_SCMOBJ {___processor ___ps=___PSTATE;int ___err=___NO_ERR;___WORD ___result;
#define ___END_CDEF_SCMOBJ if (___err) ___propagate_error (___err); return ___result; }
#define ___CDEF_CALL_SCMOBJ(lbl)___call (___NARGS,___lp+(lbl)*___LS*___WS);
#define ___CDEF_EXIT_SCMOBJ ___result = ___CDEF_RESULT;

#define ___BEGIN_CDEF(decl){___processor ___ps=___PSTATE;int ___err=___NO_ERR;decl;
#define ___END_CDEF if (___err) ___propagate_error (___err); return ___result; }
#define ___CDEF_CALL(lbl)___call (___NARGS,___lp+(lbl)*___LS*___WS);
#define ___CDEF_EXIT

#define ___BEGIN_CDEF_ARG(i,s){___WORD s;
#define ___END_CDEF_ARG(i)}
#define ___BEGIN_CDEF_ARG_SCMOBJ(i){
#define ___END_CDEF_ARG_SCMOBJ(i)}
#define ___CDEF_ARG(i,arg)___PSSTK(i)=arg;
#define ___CDEF_RESULT ___PSR1


#define ___BEGIN_CDEF_SCMOBJ_TO_CHAR(s,c)___STOC(___scmobj_to_char,s,c){
#define ___END_CDEF_SCMOBJ_TO_CHAR(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_SCHAR(s,c)___STOC(___scmobj_to_schar,s,c){
#define ___END_CDEF_SCMOBJ_TO_SCHAR(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_UCHAR(s,c)___STOC(___scmobj_to_uchar,s,c){
#define ___END_CDEF_SCMOBJ_TO_UCHAR(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_LATIN1(s,c)___STOC(___scmobj_to_latin1,s,c){
#define ___END_CDEF_SCMOBJ_TO_LATIN1(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_UCS4(s,c)___STOC(___scmobj_to_ucs4,s,c){
#define ___END_CDEF_SCMOBJ_TO_UCS4(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_UCS2(s,c)___STOC(___scmobj_to_ucs2,s,c){
#define ___END_CDEF_SCMOBJ_TO_UCS2(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_SHORT(s,c)___STOC(___scmobj_to_short,s,c){
#define ___END_CDEF_SCMOBJ_TO_SHORT(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_USHORT(s,c)___STOC(___scmobj_to_ushort,s,c){
#define ___END_CDEF_SCMOBJ_TO_USHORT(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_INT(s,c)___STOC(___scmobj_to_int,s,c){
#define ___END_CDEF_SCMOBJ_TO_INT(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_UINT(s,c)___STOC(___scmobj_to_uint,s,c){
#define ___END_CDEF_SCMOBJ_TO_UINT(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_LONG(s,c)___STOC(___scmobj_to_long,s,c){
#define ___END_CDEF_SCMOBJ_TO_LONG(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_ULONG(s,c)___STOC(___scmobj_to_ulong,s,c){
#define ___END_CDEF_SCMOBJ_TO_ULONG(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_FLOAT(s,c)___STOC(___scmobj_to_float,s,c){
#define ___END_CDEF_SCMOBJ_TO_FLOAT(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_DOUBLE(s,c)___STOC(___scmobj_to_double,s,c){
#define ___END_CDEF_SCMOBJ_TO_DOUBLE(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_POINTER(s,c)___STOC(___scmobj_to_pointer,s,c){
#define ___END_CDEF_SCMOBJ_TO_POINTER(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_FUNCTION(s,c)___STOC(___scmobj_to_function,s,c){
#define ___END_CDEF_SCMOBJ_TO_FUNCTION(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_BOOLEAN(s,c)___STOC(___scmobj_to_boolean,s,c){
#define ___END_CDEF_SCMOBJ_TO_BOOLEAN(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_CHARSTRING(s,c)___STOC(___scmobj_to_charstring,s,c){
#define ___END_CDEF_SCMOBJ_TO_CHARSTRING(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_LATIN1STRING(s,c)___STOC(___scmobj_to_latin1string,s,c){
#define ___END_CDEF_SCMOBJ_TO_LATIN1STRING(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_UCS4STRING(s,c)___STOC(___scmobj_to_ucs4string,s,c){
#define ___END_CDEF_SCMOBJ_TO_UCS4STRING(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_UCS2STRING(s,c)___STOC(___scmobj_to_ucs2string,s,c){
#define ___END_CDEF_SCMOBJ_TO_UCS2STRING(s,c)}
#define ___BEGIN_CDEF_SCMOBJ_TO_UTF8STRING(s,c)___STOC(___scmobj_to_utf8string,s,c){
#define ___END_CDEF_SCMOBJ_TO_UTF8STRING(s,c)}

#define ___BEGIN_CDEF_CHAR_TO_SCMOBJ(c,s)___CTOS(___char_to_scmobj,c,s){
#define ___END_CDEF_CHAR_TO_SCMOBJ(c,s)}
#define ___BEGIN_CDEF_SCHAR_TO_SCMOBJ(c,s)___CTOS(___schar_to_scmobj,c,s){
#define ___END_CDEF_SCHAR_TO_SCMOBJ(c,s)}
#define ___BEGIN_CDEF_UCHAR_TO_SCMOBJ(c,s)___CTOS(___uchar_to_scmobj,c,s){
#define ___END_CDEF_UCHAR_TO_SCMOBJ(c,s)}
#define ___BEGIN_CDEF_LATIN1_TO_SCMOBJ(c,s)___CTOS(___latin1_to_scmobj,c,s){
#define ___END_CDEF_LATIN1_TO_SCMOBJ(c,s)}
#define ___BEGIN_CDEF_UCS4_TO_SCMOBJ(c,s)___CTOS(___ucs4_to_scmobj,c,s){
#define ___END_CDEF_UCS4_TO_SCMOBJ(c,s)}
#define ___BEGIN_CDEF_UCS2_TO_SCMOBJ(c,s)___CTOS(___ucs2_to_scmobj,c,s){
#define ___END_CDEF_UCS2_TO_SCMOBJ(c,s)}
#define ___BEGIN_CDEF_SHORT_TO_SCMOBJ(c,s)___CTOS(___short_to_scmobj,c,s){
#define ___END_CDEF_SHORT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_USHORT_TO_SCMOBJ(c,s)___CTOS(___ushort_to_scmobj,c,s){
#define ___END_CDEF_USHORT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_INT_TO_SCMOBJ(c,s)___CTOS(___int_to_scmobj,c,s){
#define ___END_CDEF_INT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_UINT_TO_SCMOBJ(c,s)___CTOS(___uint_to_scmobj,c,s){
#define ___END_CDEF_UINT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_LONG_TO_SCMOBJ(c,s)___CTOS(___long_to_scmobj,c,s){
#define ___END_CDEF_LONG_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_ULONG_TO_SCMOBJ(c,s)___CTOS(___ulong_to_scmobj,c,s){
#define ___END_CDEF_ULONG_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_FLOAT_TO_SCMOBJ(c,s)___CTOS(___float_to_scmobj,c,s){
#define ___END_CDEF_FLOAT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_DOUBLE_TO_SCMOBJ(c,s)___CTOS(___double_to_scmobj,c,s){
#define ___END_CDEF_DOUBLE_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_POINTER_TO_SCMOBJ(c,s)___CTOS(___pointer_to_scmobj,c,s){
#define ___END_CDEF_POINTER_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_FUNCTION_TO_SCMOBJ(c,s)___CTOS(___function_to_scmobj,c,s){
#define ___END_CDEF_FUNCTION_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_BOOLEAN_TO_SCMOBJ(c,s)___CTOS(___boolean_to_scmobj,c,s){
#define ___END_CDEF_BOOLEAN_TO_SCMOBJ(c,s)}
#define ___BEGIN_CDEF_CHARSTRING_TO_SCMOBJ(c,s)___CTOS(___charstring_to_scmobj,c,s){
#define ___END_CDEF_CHARSTRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_LATIN1STRING_TO_SCMOBJ(c,s)___CTOS(___latin1string_to_scmobj,c,s){
#define ___END_CDEF_LATIN1STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_UCS4STRING_TO_SCMOBJ(c,s)___CTOS(___ucs4string_to_scmobj,c,s){
#define ___END_CDEF_UCS4STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_UCS2STRING_TO_SCMOBJ(c,s)___CTOS(___ucs2string_to_scmobj,c,s){
#define ___END_CDEF_UCS2STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CDEF_UTF8STRING_TO_SCMOBJ(c,s)___CTOS(___utf8string_to_scmobj,c,s){
#define ___END_CDEF_UTF8STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}


#define ___BEGIN_CLAM_VOID ___W_ALL {int ___err=___NO_ERR;
#define ___END_CLAM_VOID if (___err) ___C_CONV_ERROR } ___R_ALL
#define ___CLAM_EXIT_VOID ___CLAM_RESULT = ___VOID;

#define ___BEGIN_CLAM_SCMOBJ ___W_ALL {int ___err=___NO_ERR;___WORD ___result;
#define ___END_CLAM_SCMOBJ if (___err) ___C_CONV_ERROR } ___R_ALL
#define ___CLAM_EXIT_SCMOBJ ___CLAM_RESULT = ___result;

#define ___BEGIN_CLAM(decl)___W_ALL {int ___err=___NO_ERR;decl;
#define ___END_CLAM if (___err) ___C_CONV_ERROR } ___R_ALL
#define ___CLAM_EXIT

#define ___BEGIN_CLAM_ARG(i,decl){decl;
#define ___END_CLAM_ARG(i)}
#define ___BEGIN_CLAM_ARG_SCMOBJ(i){
#define ___END_CLAM_ARG_SCMOBJ(i)}
#define ___CLAM_ARG(i)___PSSTK((i-___NARGS-1))
#define ___CLAM_RESULT ___PSR1

#define ___BEGIN_CLAM_SCMOBJ_TO_CHAR(s,c)___STOC(___scmobj_to_char,s,c){
#define ___END_CLAM_SCMOBJ_TO_CHAR(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_SCHAR(s,c)___STOC(___scmobj_to_schar,s,c){
#define ___END_CLAM_SCMOBJ_TO_SCHAR(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_UCHAR(s,c)___STOC(___scmobj_to_uchar,s,c){
#define ___END_CLAM_SCMOBJ_TO_UCHAR(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_LATIN1(s,c)___STOC(___scmobj_to_latin1,s,c){
#define ___END_CLAM_SCMOBJ_TO_LATIN1(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_UCS4(s,c)___STOC(___scmobj_to_ucs4,s,c){
#define ___END_CLAM_SCMOBJ_TO_UCS4(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_UCS2(s,c)___STOC(___scmobj_to_ucs2,s,c){
#define ___END_CLAM_SCMOBJ_TO_UCS2(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_SHORT(s,c)___STOC(___scmobj_to_short,s,c){
#define ___END_CLAM_SCMOBJ_TO_SHORT(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_USHORT(s,c)___STOC(___scmobj_to_ushort,s,c){
#define ___END_CLAM_SCMOBJ_TO_USHORT(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_INT(s,c)___STOC(___scmobj_to_int,s,c){
#define ___END_CLAM_SCMOBJ_TO_INT(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_UINT(s,c)___STOC(___scmobj_to_uint,s,c){
#define ___END_CLAM_SCMOBJ_TO_UINT(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_LONG(s,c)___STOC(___scmobj_to_long,s,c){
#define ___END_CLAM_SCMOBJ_TO_LONG(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_ULONG(s,c)___STOC(___scmobj_to_ulong,s,c){
#define ___END_CLAM_SCMOBJ_TO_ULONG(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_FLOAT(s,c)___STOC(___scmobj_to_float,s,c){
#define ___END_CLAM_SCMOBJ_TO_FLOAT(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_DOUBLE(s,c)___STOC(___scmobj_to_double,s,c){
#define ___END_CLAM_SCMOBJ_TO_DOUBLE(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_POINTER(s,c)___STOC(___scmobj_to_pointer,s,c){
#define ___END_CLAM_SCMOBJ_TO_POINTER(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_FUNCTION(s,c)___STOC(___scmobj_to_function,s,c){
#define ___END_CLAM_SCMOBJ_TO_FUNCTION(s,c)___EXT(___free_function)(c);}
#define ___BEGIN_CLAM_SCMOBJ_TO_BOOLEAN(s,c)___STOC(___scmobj_to_boolean,s,c){
#define ___END_CLAM_SCMOBJ_TO_BOOLEAN(s,c)}
#define ___BEGIN_CLAM_SCMOBJ_TO_CHARSTRING(s,c)___STOC(___scmobj_to_charstring,s,c){
#define ___END_CLAM_SCMOBJ_TO_CHARSTRING(s,c)___EXT(___free_charstring)(c);}
#define ___BEGIN_CLAM_SCMOBJ_TO_LATIN1STRING(s,c)___STOC(___scmobj_to_latin1string,s,c){
#define ___END_CLAM_SCMOBJ_TO_LATIN1STRING(s,c)___EXT(___free_latin1string)(c);}
#define ___BEGIN_CLAM_SCMOBJ_TO_UCS4STRING(s,c)___STOC(___scmobj_to_ucs4string,s,c){
#define ___END_CLAM_SCMOBJ_TO_UCS4STRING(s,c)___EXT(___free_ucs4string)(c);}
#define ___BEGIN_CLAM_SCMOBJ_TO_UCS2STRING(s,c)___STOC(___scmobj_to_ucs2string,s,c){
#define ___END_CLAM_SCMOBJ_TO_UCS2STRING(s,c)___EXT(___free_ucs2string)(c);}
#define ___BEGIN_CLAM_SCMOBJ_TO_UTF8STRING(s,c)___STOC(___scmobj_to_utf8string,s,c){
#define ___END_CLAM_SCMOBJ_TO_UTF8STRING(s,c)___EXT(___free_utf8string)(c);}

#define ___BEGIN_CLAM_CHAR_TO_SCMOBJ(c,s)___CTOS(___char_to_scmobj,c,s){
#define ___END_CLAM_CHAR_TO_SCMOBJ(c,s)}
#define ___BEGIN_CLAM_SCHAR_TO_SCMOBJ(c,s)___CTOS(___schar_to_scmobj,c,s){
#define ___END_CLAM_SCHAR_TO_SCMOBJ(c,s)}
#define ___BEGIN_CLAM_UCHAR_TO_SCMOBJ(c,s)___CTOS(___uchar_to_scmobj,c,s){
#define ___END_CLAM_UCHAR_TO_SCMOBJ(c,s)}
#define ___BEGIN_CLAM_LATIN1_TO_SCMOBJ(c,s)___CTOS(___latin1_to_scmobj,c,s){
#define ___END_CLAM_LATIN1_TO_SCMOBJ(c,s)}
#define ___BEGIN_CLAM_UCS4_TO_SCMOBJ(c,s)___CTOS(___ucs4_to_scmobj,c,s){
#define ___END_CLAM_UCS4_TO_SCMOBJ(c,s)}
#define ___BEGIN_CLAM_UCS2_TO_SCMOBJ(c,s)___CTOS(___ucs2_to_scmobj,c,s){
#define ___END_CLAM_UCS2_TO_SCMOBJ(c,s)}
#define ___BEGIN_CLAM_SHORT_TO_SCMOBJ(c,s)___CTOS(___short_to_scmobj,c,s){
#define ___END_CLAM_SHORT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_USHORT_TO_SCMOBJ(c,s)___CTOS(___ushort_to_scmobj,c,s){
#define ___END_CLAM_USHORT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_INT_TO_SCMOBJ(c,s)___CTOS(___int_to_scmobj,c,s){
#define ___END_CLAM_INT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_UINT_TO_SCMOBJ(c,s)___CTOS(___uint_to_scmobj,c,s){
#define ___END_CLAM_UINT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_LONG_TO_SCMOBJ(c,s)___CTOS(___long_to_scmobj,c,s){
#define ___END_CLAM_LONG_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_ULONG_TO_SCMOBJ(c,s)___CTOS(___ulong_to_scmobj,c,s){
#define ___END_CLAM_ULONG_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_FLOAT_TO_SCMOBJ(c,s)___CTOS(___float_to_scmobj,c,s){
#define ___END_CLAM_FLOAT_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_DOUBLE_TO_SCMOBJ(c,s)___CTOS(___double_to_scmobj,c,s){
#define ___END_CLAM_DOUBLE_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_POINTER_TO_SCMOBJ(c,s)___CTOS(___pointer_to_scmobj,c,s){
#define ___END_CLAM_POINTER_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_FUNCTION_TO_SCMOBJ(c,s)___CTOS(___function_to_scmobj,c,s){
#define ___END_CLAM_FUNCTION_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_BOOLEAN_TO_SCMOBJ(c,s)___CTOS(___boolean_to_scmobj,c,s){
#define ___END_CLAM_BOOLEAN_TO_SCMOBJ(c,s)}
#define ___BEGIN_CLAM_CHARSTRING_TO_SCMOBJ(c,s)___CTOS(___charstring_to_scmobj,c,s){
#define ___END_CLAM_CHARSTRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_LATIN1STRING_TO_SCMOBJ(c,s)___CTOS(___latin1string_to_scmobj,c,s){
#define ___END_CLAM_LATIN1STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_UCS4STRING_TO_SCMOBJ(c,s)___CTOS(___ucs4string_to_scmobj,c,s){
#define ___END_CLAM_UCS4STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_UCS2STRING_TO_SCMOBJ(c,s)___CTOS(___ucs2string_to_scmobj,c,s){
#define ___END_CLAM_UCS2STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}
#define ___BEGIN_CLAM_UTF8STRING_TO_SCMOBJ(c,s)___CTOS(___utf8string_to_scmobj,c,s){
#define ___END_CLAM_UTF8STRING_TO_SCMOBJ(c,s)___EXT(___release_scmobj)(s);}

#endif
